# !/usr/bin/env python
'''
$Rev::                                  $:  Revision of last commit
$Author::                               $:  Author of last commit
$Date::                                 $:  Date of last commit
'''
# -*- coding: UTF-8 -*-
# generated by wxGlade 0.4.1 on Tue Sep  4 21:25:55 2007

import wx
import wx.grid, wx.py, wx.stc
import io
import traceback
from logging import info, debug, warning, error

import os, sys, shutil, appdirs

# import wx.lib.agw.aui as aui

from genx import data, model, help
from genx import filehandling as io
from genx import plotpanel, solvergui, parametergrid, datalist
from genx import event_handlers
from genx import images as img
from genx.version import __version__ as program_version

from genx.plugins import add_on_framework as add_on

# Add current path to the system paths
# just in case some user make a directory change
sys.path.append(os.getcwd())
_path, _file=os.path.split(__file__)
if _path[-4:]=='.zip':
    _path, ending=os.path.split(_path)

# Get the configuration path, create if it not exists
config_path=appdirs.user_data_dir('GenX3', 'ArturGlavic')+'/'
info(config_path)
version_file=os.path.join(config_path, 'genx.version')
if not os.path.exists(config_path):
    info('Creating path: %s'%config_path)
    os.makedirs(config_path)
if not os.path.exists(os.path.join(config_path, 'profiles')):
    info('Creating path: %s'%os.path.join(config_path, 'profiles'))
    shutil.copytree(os.path.join(_path, 'profiles'),
                    os.path.join(config_path, 'profiles'))
    open(version_file, 'w').write(program_version+'\n')
elif not os.path.exists(version_file) or \
        open(version_file, 'r').read().rsplit('.', 1)[0]!=program_version.rsplit('.', 1)[0]:
    # update profiles if major version does not match
    info('Update profiles to default for GenX '+program_version)
    from glob import glob

    for fi in glob(os.path.join(_path, 'profiles', '*.conf')):
        shutil.copy2(fi, os.path.join(config_path, 'profiles'))
    open(version_file, 'w').write(program_version+'\n')
if not os.path.exists(os.path.join(config_path, 'genx.conf')):
    info('Creating genx.conf at %s by copying config from %s'%(config_path,
                                                               os.path.join(_path, 'genx.conf')))
    shutil.copyfile(os.path.join(_path, 'genx.conf'),
                    os.path.join(config_path, 'genx.conf'))

class MainFrame(wx.Frame):
    def __init__(self, parent, *args, **kwds):
        self.parent=parent
        debug('starting setup of MainFrame')

        self.flag_simulating=False
        self.simulation_queue_counter=0

        debug('setup of MainFrame - config')
        self.config=io.Config()
        self.config.load_default(os.path.join(config_path, 'genx.conf'))

        status_text=lambda event: event_handlers.status_text(self, event)

        # begin wxGlade: MainFrame.__init__
        kwds["style"]=kwds.get("style",
                               0) | wx.CAPTION | wx.CLOSE_BOX | wx.MAXIMIZE_BOX | wx.MINIMIZE_BOX | wx.RESIZE_BORDER | wx.SYSTEM_MENU
        debug('setup of MainFrame - wx.Frame\n')
        wx.Frame.__init__(self, *args, **kwds)

        try:
            dpi_scale_factor=self.GetDPIScaleFactor()
            debug("Detected DPI scale factor %s from GetDPIScaleFactor"%dpi_scale_factor)
        except AttributeError:
            dpi_scale_factor=self.GetContentScaleFactor()
        debug("Detected DPI scale factor %s from GetContentScaleFactor"%dpi_scale_factor)
        self.dpi_scale_factor=dpi_scale_factor
        wx.GetApp().dpi_scale_factor=dpi_scale_factor
        tb_bmp_size=int(32*self.dpi_scale_factor)

        debug('setup of MainFrame - menu bar')
        # Menu Bar
        self.main_frame_menubar=wx.MenuBar()
        self.mb_file=wx.Menu()
        self.main_frame_menubar.mb_new=self.mb_file.Append(wx.ID_ANY, "New...\tCtrl+N", "Creates a new model")
        self.Bind(wx.EVT_MENU, self.eh_mb_new, id=self.main_frame_menubar.mb_new.GetId())
        self.main_frame_menubar.mb_open=self.mb_file.Append(wx.ID_ANY, "Open...\tCtrl+O", "Opens an existing model")
        self.Bind(wx.EVT_MENU, self.eh_mb_open, id=self.main_frame_menubar.mb_open.GetId())
        self.main_frame_menubar.mb_save=self.mb_file.Append(wx.ID_ANY, "Save...\tCtrl+S", "Saves the current model")
        self.Bind(wx.EVT_MENU, self.eh_mb_save, id=self.main_frame_menubar.mb_save.GetId())
        self.main_frame_menubar.mb_saveas=self.mb_file.Append(wx.ID_ANY, "Save As...",
                                                              "Saves the active model with a new name")
        self.Bind(wx.EVT_MENU, self.eh_mb_saveas, id=self.main_frame_menubar.mb_saveas.GetId())
        self.mb_file.AppendSeparator()
        mb_import=wx.Menu()
        self.main_frame_menubar.mb_import_data=mb_import.Append(wx.ID_ANY, "Import Data...\tCtrl+D",
                                                                "Import data to the active data set")
        self.Bind(wx.EVT_MENU, self.eh_mb_import_data, id=self.main_frame_menubar.mb_import_data.GetId())
        self.main_frame_menubar.mb_import_table=mb_import.Append(wx.ID_ANY, "Import Table...",
                                                                 "Import a table from an ASCII file")
        self.Bind(wx.EVT_MENU, self.eh_mb_import_table, id=self.main_frame_menubar.mb_import_table.GetId())
        self.main_frame_menubar.mb_import_script=mb_import.Append(wx.ID_ANY, "Import Script...",
                                                                  "Import a python model script")
        self.Bind(wx.EVT_MENU, self.eh_mb_import_script, id=self.main_frame_menubar.mb_import_script.GetId())
        self.mb_file.Append(wx.ID_ANY, "Import", mb_import, "")
        mb_export=wx.Menu()
        self.main_frame_menubar.mb_export_orso=mb_export.Append(wx.ID_ANY, "Export ORT (alpha)...",
                                                                "Export data and header in ORSO compatible ASCII format")
        self.Bind(wx.EVT_MENU, self.eh_mb_export_orso, id=self.main_frame_menubar.mb_export_orso.GetId())
        self.main_frame_menubar.mb_export_data=mb_export.Append(wx.ID_ANY, "Export Data...",
                                                                "Export data in ASCII format")
        self.Bind(wx.EVT_MENU, self.eh_mb_export_data, id=self.main_frame_menubar.mb_export_data.GetId())
        self.main_frame_menubar.mb_export_table=mb_export.Append(wx.ID_ANY, "Export Table...",
                                                                 "Export table to an ASCII file")
        self.Bind(wx.EVT_MENU, self.eh_mb_export_table, id=self.main_frame_menubar.mb_export_table.GetId())
        self.main_frame_menubar.mb_export_script=mb_export.Append(wx.ID_ANY, "Export Script...",
                                                                  "Export the script to a python file")
        self.Bind(wx.EVT_MENU, self.eh_mb_export_script, id=self.main_frame_menubar.mb_export_script.GetId())
        self.mb_file.Append(wx.ID_ANY, "Export", mb_export, "")
        self.mb_file.AppendSeparator()
        mb_print=wx.Menu()
        self.main_frame_menubar.mb_print_plot=mb_print.Append(wx.ID_ANY, "Print Plot...\tCtrl+P",
                                                              "Print the current plot")
        self.Bind(wx.EVT_MENU, self.eh_mb_print_plot, id=self.main_frame_menubar.mb_print_plot.GetId())
        self.main_frame_menubar.mb_print_grid=mb_print.Append(wx.ID_ANY, "Print Grid...", "Prints the grid")
        self.Bind(wx.EVT_MENU, self.eh_mb_print_grid, id=self.main_frame_menubar.mb_print_grid.GetId())
        self.main_frame_menubar.mb_print_script=mb_print.Append(wx.ID_ANY, "Print Script...", "Prints the model script")
        self.Bind(wx.EVT_MENU, self.eh_mb_print_script, id=self.main_frame_menubar.mb_print_script.GetId())
        self.mb_file.Append(wx.ID_ANY, "Print", mb_print, "")
        self.mb_file.AppendSeparator()
        self.main_frame_menubar.mb_quit=self.mb_file.Append(wx.ID_ANY, "&Quit\tAlt+Q", "Quit the program")
        self.Bind(wx.EVT_MENU, self.eh_mb_quit, id=self.main_frame_menubar.mb_quit.GetId())
        self.main_frame_menubar.Append(self.mb_file, "File")
        self.mb_edit=wx.Menu()
        self.main_frame_menubar.mb_copy_graph=self.mb_edit.Append(wx.ID_ANY, "Copy Graph",
                                                                  "Copy the current graph to the clipboard as a bitmap")
        self.Bind(wx.EVT_MENU, self.eh_mb_copy_graph, id=self.main_frame_menubar.mb_copy_graph.GetId())
        self.main_frame_menubar.mb_copy_sim=self.mb_edit.Append(wx.ID_ANY, "Copy Simulation",
                                                                "Copy the current simulation and data as ASCII text")
        self.Bind(wx.EVT_MENU, self.eh_mb_copy_sim, id=self.main_frame_menubar.mb_copy_sim.GetId())
        self.main_frame_menubar.mb_copy_table=self.mb_edit.Append(wx.ID_ANY, "Copy Table", "Copy the parameter grid")
        self.Bind(wx.EVT_MENU, self.eh_mb_copy_table, id=self.main_frame_menubar.mb_copy_table.GetId())
        self.main_frame_menubar.mb_findreplace=self.mb_edit.Append(wx.ID_ANY, "&Find/Replace...\tCtrl+F",
                                                                   "Find and replace in the script")
        self.Bind(wx.EVT_MENU, self.eh_mb_findreplace, id=self.main_frame_menubar.mb_findreplace.GetId())
        self.mb_edit_sub=wx.Menu()
        self.main_frame_menubar.mb_new_data_set=self.mb_edit_sub.Append(wx.ID_ANY, "&New data set\tAlt+N",
                                                                        "Appends a new data set")
        self.Bind(wx.EVT_MENU, self.eh_data_new_set, id=self.main_frame_menubar.mb_new_data_set.GetId())
        self.main_frame_menubar.mb_data_delete=self.mb_edit_sub.Append(wx.ID_ANY, "&Delete\tAlt+D",
                                                                       "Deletes the selected data sets")
        self.Bind(wx.EVT_MENU, self.eh_data_delete, id=self.main_frame_menubar.mb_data_delete.GetId())
        self.main_frame_menubar.mb_data_move_down=self.mb_edit_sub.Append(wx.ID_ANY, "&Lower item\tAlt+L",
                                                                          "Move selected item down")
        self.Bind(wx.EVT_MENU, self.eh_data_move_down, id=self.main_frame_menubar.mb_data_move_down.GetId())
        self.main_frame_menubar.mb_data_move_up=self.mb_edit_sub.Append(wx.ID_ANY, "&Raise item\tAlt+R",
                                                                        "Moves selected data sets up")
        self.Bind(wx.EVT_MENU, self.eh_data_move_up, id=self.main_frame_menubar.mb_data_move_up.GetId())
        self.mb_edit_sub.AppendSeparator()
        self.main_frame_menubar.mb_toggle_show=self.mb_edit_sub.Append(wx.ID_ANY, "Toggle &Show\tAlt+S",
                                                                       "Toggle show on and off for the selected data set")
        self.Bind(wx.EVT_MENU, self.eh_data_toggle_show, id=self.main_frame_menubar.mb_toggle_show.GetId())
        self.main_frame_menubar.mb_toggle_use=self.mb_edit_sub.Append(wx.ID_ANY, "Toggle &Use\tAlt+U",
                                                                      "Toggle use on and off for the selected data sets")
        self.Bind(wx.EVT_MENU, self.eh_data_toggle_use, id=self.main_frame_menubar.mb_toggle_use.GetId())
        self.main_frame_menubar.mb_toggle_error=self.mb_edit_sub.Append(wx.ID_ANY, "Toggle &Error\tAlt+E",
                                                                        "Turn the use of error on and off")
        self.Bind(wx.EVT_MENU, self.eh_data_toggle_error, id=self.main_frame_menubar.mb_toggle_error.GetId())
        self.mb_edit_sub.AppendSeparator()
        self.main_frame_menubar.mb_toggle_calc=self.mb_edit_sub.Append(wx.ID_ANY, "&Calculations\tAlt+C",
                                                                       "OPens dialog box to define dataset calculations")
        self.Bind(wx.EVT_MENU, self.eh_data_calc, id=self.main_frame_menubar.mb_toggle_calc.GetId())
        self.mb_edit.Append(wx.ID_ANY, "Data", self.mb_edit_sub, "")
        self.main_frame_menubar.Append(self.mb_edit, "Edit")
        self.mb_view=wx.Menu()
        self.main_frame_menubar.mb_view_grid_slider=self.mb_view.Append(wx.ID_ANY, "Value as slider",
                                                                        "View and control the value as a slider",
                                                                        wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.eh_mb_view_grid_slider, id=self.main_frame_menubar.mb_view_grid_slider.GetId())
        self.main_frame_menubar.mb_view_zoom=self.mb_view.Append(wx.ID_ANY, "Zoom\tCtrl+Z", "Turn the zoom on/off",
                                                                 wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.eh_mb_view_zoom, id=self.main_frame_menubar.mb_view_zoom.GetId())
        self.main_frame_menubar.mb_view_zoomall=self.mb_view.Append(wx.ID_ANY, "Zoom All\tCtrl+A",
                                                                    "Zoom to fit all data points")
        self.Bind(wx.EVT_MENU, self.eh_mb_view_zoomall, id=self.main_frame_menubar.mb_view_zoomall.GetId())
        mb_view_yscale=wx.Menu()
        self.main_frame_menubar.mb_view_yscale_log=mb_view_yscale.Append(wx.ID_ANY, "log", "Set y-scale logarithmic",
                                                                         wx.ITEM_RADIO)
        self.Bind(wx.EVT_MENU, self.eh_mb_view_yscale_log, id=self.main_frame_menubar.mb_view_yscale_log.GetId())
        self.main_frame_menubar.mb_view_yscale_lin=mb_view_yscale.Append(wx.ID_ANY, "lin", "Set y-scale linear",
                                                                         wx.ITEM_RADIO)
        self.Bind(wx.EVT_MENU, self.eh_mb_view_yscale_linear, id=self.main_frame_menubar.mb_view_yscale_lin.GetId())
        self.mb_view.Append(wx.ID_ANY, "y scale", mb_view_yscale, "")
        mb_view_xscale=wx.Menu()
        self.main_frame_menubar.mb_view_xscale_log=mb_view_xscale.Append(wx.ID_ANY, "log", "Set x-scale logarithmic",
                                                                         wx.ITEM_RADIO)
        self.Bind(wx.EVT_MENU, self.eh_mb_view_xscale_log, id=self.main_frame_menubar.mb_view_xscale_log.GetId())
        self.main_frame_menubar.mb_view_xscale_lin=mb_view_xscale.Append(wx.ID_ANY, "lin", "Set x-scale linear",
                                                                         wx.ITEM_RADIO)
        self.Bind(wx.EVT_MENU, self.eh_mb_view_xscale_linear, id=self.main_frame_menubar.mb_view_xscale_lin.GetId())
        self.mb_view.Append(wx.ID_ANY, "x scale", mb_view_xscale, "")
        self.main_frame_menubar.mb_view_autoscale=self.mb_view.Append(wx.ID_ANY, "Autoscale",
                                                                      "Sets autoscale on when plotting", wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.eh_mb_view_autoscale, id=self.main_frame_menubar.mb_view_autoscale.GetId())
        self.main_frame_menubar.mb_use_toggle_show=self.mb_view.Append(wx.ID_ANY, "Use Toggle Show",
                                                                       "Set if the plotted data shold be toggled or selcted by the mouse",
                                                                       wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.eh_mb_view_use_toggle_show, id=self.main_frame_menubar.mb_use_toggle_show.GetId())
        self.main_frame_menubar.Append(self.mb_view, "View")
        self.mb_fit=wx.Menu()
        self.main_frame_menubar.mb_fit_simulate=self.mb_fit.Append(wx.ID_ANY, "&Simulate\tF9",
                                                                   "Compile the script and run the Sim function")
        self.Bind(wx.EVT_MENU, self.eh_tb_simulate, id=self.main_frame_menubar.mb_fit_simulate.GetId())
        self.main_frame_menubar.mb_fit_evaluate=self.mb_fit.Append(wx.ID_ANY, "&Evaluate\tF5",
                                                                   "Evaluate the Sim function only - no recompiling")
        self.Bind(wx.EVT_MENU, self.eh_mb_fit_evaluate, id=self.main_frame_menubar.mb_fit_evaluate.GetId())
        self.main_frame_menubar.mb_use_cuda=self.mb_fit.Append(wx.ID_ANY, "Use CUDA",
                                                               "Make use of Nvidia GPU computing with CUDA",
                                                               wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.eh_mb_use_cuda, id=self.main_frame_menubar.mb_use_cuda.GetId())
        self.mb_fit.AppendSeparator()
        self.main_frame_menubar.mb_fit_start=self.mb_fit.Append(wx.ID_ANY, "Start &Fit\tCtrl+F", "Start fitting")
        self.Bind(wx.EVT_MENU, self.eh_mb_fit_start, id=self.main_frame_menubar.mb_fit_start.GetId())
        self.main_frame_menubar.mb_fit_stop=self.mb_fit.Append(wx.ID_ANY, "&Halt Fit\tCtrl+H", "Stop fitting")
        self.Bind(wx.EVT_MENU, self.eh_mb_fit_stop, id=self.main_frame_menubar.mb_fit_stop.GetId())
        self.main_frame_menubar.mb_fit_resume=self.mb_fit.Append(wx.ID_ANY, "&Resume Fit\tCtrl+R",
                                                                 "Resumes fitting without reinitilazation of the optimizer")
        self.Bind(wx.EVT_MENU, self.eh_mb_fit_resume, id=self.main_frame_menubar.mb_fit_resume.GetId())
        self.main_frame_menubar.mb_fit_analyze=self.mb_fit.Append(wx.ID_ANY, "Analyze fit", "Analyze the fit")
        self.Bind(wx.EVT_MENU, self.eh_mb_fit_analyze, id=self.main_frame_menubar.mb_fit_analyze.GetId())
        self.mb_fit.AppendSeparator()
        self.main_frame_menubar.mb_fit_autosim=self.mb_fit.Append(wx.ID_ANY, "Simulate Automatically",
                                                                  "Update simulation on model changes automatically",
                                                                  wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.eh_mb_fit_autosim, id=self.main_frame_menubar.mb_fit_autosim.GetId())
        self.main_frame_menubar.Append(self.mb_fit, "Fit")
        self.mb_set=wx.Menu()
        mb_set_plugins=wx.Menu()
        mb_set_plugins.AppendSeparator()
        self.mb_set.Append(wx.ID_ANY, "Plugins", mb_set_plugins, "")
        self.main_frame_menubar.mb_set_opt=self.mb_set.Append(wx.ID_ANY, "Optimizer\tShift+Ctrl+O", "")
        self.Bind(wx.EVT_MENU, self.eh_mb_set_opt, id=self.main_frame_menubar.mb_set_opt.GetId())
        self.main_frame_menubar.mb_set_dataloader=self.mb_set.Append(wx.ID_ANY, "Data Loader\tShift+Ctrl+D", "")
        self.Bind(wx.EVT_MENU, self.eh_mb_set_dal, id=self.main_frame_menubar.mb_set_dataloader.GetId())
        self.main_frame_menubar.mb_set_import=self.mb_set.Append(wx.ID_ANY, "Import\tShift+Ctrl+I",
                                                                 "Import settings for the data sets")
        self.Bind(wx.EVT_MENU, self.eh_data_import, id=self.main_frame_menubar.mb_set_import.GetId())
        self.main_frame_menubar.mb_set_dataplot=self.mb_set.Append(wx.ID_ANY, "Plot Markers\tShift+Ctrl+P",
                                                                   "Set the symbols and lines of data and simulations")
        self.Bind(wx.EVT_MENU, self.eh_data_plots, id=self.main_frame_menubar.mb_set_dataplot.GetId())
        self.main_frame_menubar.show_startup_dialog=self.mb_set.Append(wx.ID_ANY, "Startup Profile...", "")
        self.Bind(wx.EVT_MENU, self.eh_show_startup_dialog, id=self.main_frame_menubar.show_startup_dialog.GetId())
        self.main_frame_menubar.Append(self.mb_set, "Settings")
        wxglade_tmp_menu=wx.Menu()
        self.main_frame_menubar.mb_models_help=wxglade_tmp_menu.Append(wx.ID_ANY, "Models Help...",
                                                                       "Show help for the models")
        self.Bind(wx.EVT_MENU, self.eh_mb_models_help, id=self.main_frame_menubar.mb_models_help.GetId())
        self.main_frame_menubar.mb_fom_help=wxglade_tmp_menu.Append(wx.ID_ANY, "FOM Help", "Show help about the fom")
        self.Bind(wx.EVT_MENU, self.eh_mb_fom_help, id=self.main_frame_menubar.mb_fom_help.GetId())
        self.main_frame_menubar.mb_plugins_help=wxglade_tmp_menu.Append(wx.ID_ANY, "Plugins Helps...",
                                                                        "Show help for the plugins")
        self.Bind(wx.EVT_MENU, self.eh_mb_plugins_help, id=self.main_frame_menubar.mb_plugins_help.GetId())
        self.main_frame_menubar.mb_data_loaders_help=wxglade_tmp_menu.Append(wx.ID_ANY, "Data loaders Help...",
                                                                             "Show help for the data loaders")
        self.Bind(wx.EVT_MENU, self.eh_mb_data_loaders_help, id=self.main_frame_menubar.mb_data_loaders_help.GetId())
        wxglade_tmp_menu.AppendSeparator()
        self.main_frame_menubar.mb_misc_showman=wxglade_tmp_menu.Append(wx.ID_ANY, "Open Manual...", "Show the manual")
        self.Bind(wx.EVT_MENU, self.eh_mb_misc_showman, id=self.main_frame_menubar.mb_misc_showman.GetId())
        self.main_frame_menubar.mb_open_homepage=wxglade_tmp_menu.Append(wx.ID_ANY, "Open Homepage...",
                                                                         "Open the homepage")
        self.Bind(wx.EVT_MENU, self.eh_mb_misc_openhomepage, id=self.main_frame_menubar.mb_open_homepage.GetId())
        self.main_frame_menubar.mb_misc_about=wxglade_tmp_menu.Append(wx.ID_ANY, "About...",
                                                                      "Shows information about GenX")
        self.Bind(wx.EVT_MENU, self.eh_mb_misc_about, id=self.main_frame_menubar.mb_misc_about.GetId())
        self.main_frame_menubar.Append(wxglade_tmp_menu, "Info")
        self.SetMenuBar(self.main_frame_menubar)
        # Menu Bar end
        self.main_frame_statusbar=self.CreateStatusBar(3)

        debug('setup of MainFrame - tool bar')
        # Tool Bar
        self.main_frame_toolbar=wx.ToolBar(self, -1, style=wx.TB_DEFAULT_STYLE)
        self.SetToolBar(self.main_frame_toolbar)
        self.main_frame_toolbar.AddTool(10001, "tb_new", wx.Bitmap(img.getnewImage().Scale(tb_bmp_size, tb_bmp_size)),
                                        wx.NullBitmap, wx.ITEM_NORMAL, "New model | Ctrl+N",
                                        "Create a new model | Ctrl+N")
        self.main_frame_toolbar.AddTool(10002, "tb_open", wx.Bitmap(img.getopenImage().Scale(tb_bmp_size, tb_bmp_size)),
                                        wx.NullBitmap, wx.ITEM_NORMAL, "Open | Ctrl+O",
                                        "Open an existing model | Ctrl+O")
        self.main_frame_toolbar.AddTool(10003, "tb_save", wx.Bitmap(img.getsaveImage().Scale(tb_bmp_size, tb_bmp_size)),
                                        wx.NullBitmap, wx.ITEM_NORMAL, "Save | Ctrl+S", "Save model to file | Ctrl+S")
        self.main_frame_toolbar.AddSeparator()
        self.main_frame_toolbar.AddTool(10004, "tb_simulate",
                                        wx.Bitmap(img.getsimulateImage().Scale(tb_bmp_size, tb_bmp_size)),
                                        wx.NullBitmap, wx.ITEM_NORMAL, "Simulate | F9", "Simulate the model | F9")
        self.main_frame_toolbar.AddTool(10005, "tb_start_fit",
                                        wx.Bitmap(img.getstart_fitImage().Scale(tb_bmp_size, tb_bmp_size)),
                                        wx.NullBitmap, wx.ITEM_NORMAL, "Start fit | Ctrl+F", "Start fitting | Ctrl+F")
        self.main_frame_toolbar.AddTool(10006, "tb_stop_fit",
                                        wx.Bitmap(img.getstop_fitImage().Scale(tb_bmp_size, tb_bmp_size)),
                                        wx.NullBitmap, wx.ITEM_NORMAL, "Stop fit | Ctrl+H", "Stop fitting | Ctrl+H")
        self.main_frame_toolbar.AddTool(10007, "tb_restart_fit",
                                        wx.Bitmap(img.getrestart_fitImage().Scale(tb_bmp_size, tb_bmp_size)),
                                        wx.NullBitmap, wx.ITEM_NORMAL, "Restart fit | Ctrl+R",
                                        "Restart the fit | Ctrl+R")
        self.main_frame_toolbar.AddTool(1008, "tb_calc_error_bars",
                                        wx.Bitmap(img.getcalc_error_barImage().Scale(tb_bmp_size, tb_bmp_size)),
                                        wx.NullBitmap, wx.ITEM_NORMAL, "Calculate errorbars", "Calculate errorbars")
        self.main_frame_toolbar.AddTool(10010, "tb_error_stats",
                                        wx.Bitmap(img.getpar_projImage().Scale(tb_bmp_size, tb_bmp_size)),
                                        wx.NullBitmap, wx.ITEM_NORMAL, "Error Statistics", "Error Statistics")
        self.main_frame_toolbar.AddSeparator()
        self.main_frame_toolbar.AddTool(10009, "tb_zoom", wx.Bitmap(img.getzoomImage().Scale(tb_bmp_size, tb_bmp_size)),
                                        wx.NullBitmap, wx.ITEM_CHECK, "Zoom | Ctrl+Z", "Turn zoom on/off  | Ctrl+Z")
        # Tool Bar end
        debug('setup of MainFrame - splitters and panels')
        self.ver_splitter=wx.SplitterWindow(self, wx.ID_ANY, style=wx.SP_3D | wx.SP_BORDER | wx.SP_LIVE_UPDATE)
        self.data_panel=wx.Panel(self.ver_splitter, wx.ID_ANY)
        self.data_notebook=wx.Notebook(self.data_panel, wx.ID_ANY)
        self.data_notebook_data=wx.Panel(self.data_notebook, wx.ID_ANY)
        self.data_list=datalist.DataListControl(self.data_notebook_data, wx.ID_ANY, self.config, status_text)
        self.data_notebook_pane_2=wx.Panel(self.data_notebook, wx.ID_ANY)
        self.label_2=wx.StaticText(self.data_notebook_pane_2, wx.ID_ANY, "  Data set: ")
        self.data_grid_choice=wx.Choice(self.data_notebook_pane_2, wx.ID_ANY, choices=["test2", "test1"])
        self.static_line_1=wx.StaticLine(self.data_notebook_pane_2, wx.ID_ANY)
        self.data_grid=wx.grid.Grid(self.data_notebook_pane_2, wx.ID_ANY, size=(1, 1))
        self.main_panel=wx.Panel(self.ver_splitter, wx.ID_ANY)
        self.hor_splitter=wx.SplitterWindow(self.main_panel, wx.ID_ANY,
                                            style=wx.SP_3D | wx.SP_BORDER | wx.SP_LIVE_UPDATE)
        self.plot_panel=wx.Panel(self.hor_splitter, wx.ID_ANY)
        self.plot_splitter=wx.SplitterWindow(self.plot_panel, wx.ID_ANY)
        self.plot_notebook=wx.Notebook(self.plot_splitter, wx.ID_ANY, style=wx.NB_BOTTOM)
        self.plot_notebook_data=wx.Panel(self.plot_notebook, wx.ID_ANY)
        self.plot_data=plotpanel.DataPlotPanel(self.plot_notebook_data, config=self.config, config_name='data plot')
        self.plot_notebook_fom=wx.Panel(self.plot_notebook, wx.ID_ANY)
        self.plot_fom=plotpanel.ErrorPlotPanel(self.plot_notebook_fom, config=self.config, config_name='fom plot')
        self.plot_notebook_Pars=wx.Panel(self.plot_notebook, wx.ID_ANY)
        self.plot_pars=plotpanel.ParsPlotPanel(self.plot_notebook_Pars, config=self.config, config_name='pars plot')
        self.plot_notebook_foms=wx.Panel(self.plot_notebook, wx.ID_ANY)
        self.plot_fomscan=plotpanel.FomScanPlotPanel(self.plot_notebook_foms, config=self.config,
                                                     config_name='fom scan plot')
        self.wide_plugin_notebook=wx.Notebook(self.plot_splitter, wx.ID_ANY, style=wx.NB_BOTTOM)
        self.panel_1=wx.Panel(self.wide_plugin_notebook, wx.ID_ANY)
        self.input_panel=wx.Panel(self.hor_splitter, wx.ID_ANY)
        self.input_notebook=wx.Notebook(self.input_panel, wx.ID_ANY, style=wx.NB_BOTTOM)
        self.input_notebook_grid=wx.Panel(self.input_notebook, wx.ID_ANY)
        self.paramter_grid=parametergrid.ParameterGrid(self.input_notebook_grid, self, config=self.config)
        self.input_notebook_script=wx.Panel(self.input_notebook, wx.ID_ANY)
        self.script_editor=wx.py.editwindow.EditWindow(self.input_notebook_script, wx.ID_ANY)
        self.script_editor.SetBackSpaceUnIndents(True)
        self.script_editor.Bind(wx.EVT_KEY_DOWN, self.ScriptEditorKeyEvent)

        debug('setup of MainFrame - properties and layout')
        self.__set_properties()
        self.__do_layout()

        debug('setup of MainFrame - bind')
        self.Bind(wx.EVT_TOOL, self.eh_tb_new, id=10001)
        self.Bind(wx.EVT_TOOL, self.eh_tb_open, id=10002)
        self.Bind(wx.EVT_TOOL, self.eh_tb_save, id=10003)
        self.Bind(wx.EVT_TOOL, self.eh_tb_simulate, id=10004)
        self.Bind(wx.EVT_TOOL, self.eh_tb_start_fit, id=10005)
        self.Bind(wx.EVT_TOOL, self.eh_tb_stop_fit, id=10006)
        self.Bind(wx.EVT_TOOL, self.eh_tb_restart_fit, id=10007)
        self.Bind(wx.EVT_TOOL, self.eh_tb_calc_error_bars, id=1008)
        self.Bind(wx.EVT_TOOL, self.eh_tb_error_stats, id=10010)
        self.Bind(wx.EVT_TOOL, self.eh_tb_zoom, id=10009)
        self.Bind(wx.EVT_CHOICE, self.eh_data_grid_choice, self.data_grid_choice)
        self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.eh_plot_page_changed, self.plot_notebook)
        # end wxGlade

        debug('setup of MainFrame - manual config')
        #### Begin Manual Config

        # Create objects needed
        # Set the style on the toolbar to
        # style=wx.TB_HORIZONTAL|wx.TB_FLAT
        # self.main_frame_fom_text = wx.StaticText(self.main_frame_toolbar, -1,\
        #    '        FOM: ', size = (200, -1))
        # font = wx.Font(15, wx.FONTFAMILY_DEFAULT, wx.NORMAL, wx.NORMAL)
        # self.main_frame_fom_text.SetFont(font)
        # self.main_frame_fom_text.SetLabel('        FOM: None')
        ##self.main_frame_fom_text.SetEditable(False)
        # self.main_frame_toolbar.AddSeparator()
        # self.main_frame_toolbar.AddControl(self.main_frame_fom_text)

        self.model=model.Model(config=self.config)
        self.model.data=self.data_list.data_cont.data
        self.paramter_grid.SetParameters(self.model.parameters)

        if self.model.script!='':
            self.script_editor.SetText(self.model.script)
        self.solver_control=solvergui.SolverController(self, self.config)

        self.plugin_control= \
            add_on.PluginController(self, mb_set_plugins, self.config)

        # Bind all the events that are needed to occur when a new model has
        # been loaded
        # Update the parameter grid
        self.Bind(event_handlers.EVT_NEW_MODEL, self.paramter_grid.OnNewModel, self)
        self.Bind(event_handlers.EVT_NEW_MODEL, self.data_list.eh_external_new_model, self)
        # Update the script
        self.Bind(event_handlers.EVT_NEW_MODEL, self.eh_new_model, self)
        # Event that the plot should respond to
        self.Bind(datalist.EVT_DATA_LIST, self.plot_data.OnDataListEvent, self.data_list.list_ctrl)
        self.Bind(datalist.EVT_DATA_LIST, self.eh_external_update_data_grid_choice, self.data_list.list_ctrl)
        self.Bind(datalist.EVT_DATA_LIST, self.eh_external_update_data, self.data_list.list_ctrl)

        self.Bind(event_handlers.EVT_SIM_PLOT, self.plot_data.OnSimPlotEvent,
                  self)
        self.Bind(event_handlers.EVT_SIM_PLOT, self.eh_external_fom_value,
                  self)
        # Update events from the solver
        self.Bind(solvergui.EVT_UPDATE_PLOT, self.eh_external_fom_value)
        self.Bind(solvergui.EVT_UPDATE_PLOT, self.plot_data.OnSolverPlotEvent)
        self.Bind(solvergui.EVT_UPDATE_PLOT, self.plot_fom.OnSolverPlotEvent)

        self.Bind(solvergui.EVT_SOLVER_UPDATE_TEXT,
                  self.eh_ex_status_text)
        self.Bind(solvergui.EVT_UPDATE_PARAMETERS,
                  self.paramter_grid.OnSolverUpdateEvent)
        self.Bind(solvergui.EVT_UPDATE_PARAMETERS,
                  self.plot_pars.OnSolverParameterEvent)
        # For picking a point in a plot
        self.Bind(plotpanel.EVT_PLOT_POSITION,
                  self.eh_ex_point_pick)
        # This is needed to be able to create the events
        self.plot_data.SetCallbackWindow(self)
        self.plot_fom.SetCallbackWindow(self)
        self.plot_pars.SetCallbackWindow(self)
        self.plot_fomscan.SetCallbackWindow(self)
        self.Bind(plotpanel.EVT_PLOT_SETTINGS_CHANGE,
                  self.eh_ex_plot_settings_changed)

        # Binding events which means model changes
        self.Bind(parametergrid.EVT_PARAMETER_GRID_CHANGE,
                  self.eh_external_model_changed)
        self.Bind(wx.stc.EVT_STC_MODIFIED, self.eh_external_model_changed,
                  self.script_editor)
        self.Bind(datalist.EVT_DATA_LIST, self.eh_external_model_changed,
                  self.data_list.list_ctrl)

        # Event for when a value of a parameter in the parameter grid has been updated
        self.Bind(parametergrid.EVT_PARAMETER_VALUE_CHANGE,
                  self.eh_external_parameter_value_changed)

        # Stuff for the find and replace functionallity
        self.findreplace_data=wx.FindReplaceData()
        # Make search down as default
        self.findreplace_data.SetFlags(1)
        self.findreplace_dlg=wx.FindReplaceDialog(self,
                                                  self.findreplace_data,
                                                  "Find & replace",
                                                  wx.FR_REPLACEDIALOG)
        self.Bind(wx.EVT_FIND, self.eh_external_find)
        self.Bind(wx.EVT_FIND_NEXT, self.eh_external_find)
        self.Bind(wx.EVT_FIND_REPLACE, self.eh_external_find)
        self.Bind(wx.EVT_FIND_REPLACE_ALL, self.eh_external_find)
        self.Bind(wx.EVT_FIND_CLOSE, self.eh_external_find)

        # Adding close event so I can take care of it...
        self.Bind(wx.EVT_CLOSE, self.eh_mb_quit)

        proj_func=lambda row: event_handlers.project_fom_parameter(self, row)
        scan_func=lambda row: event_handlers.scan_parameter(self, row)
        self.paramter_grid.SetFOMFunctions(proj_func, scan_func)

        # Initiializations..
        # To force an update of the menubar...
        self.plot_data.SetZoom(False)

        try:
            for p in [self.plot_data, self.plot_fom,
                      self.plot_pars, self.plot_fomscan]:
                p.ReadConfig()
        except Exception as e:
            outp=io.StringIO()
            traceback.print_exc(200, outp)
            val=outp.getvalue()
            outp.close()
            error('Error in loading config for the plots. Pyton tractback:\n %s'%val)
            event_handlers.ShowErrorDialog(self, 'Could not read the config for the plots. Python Error:\n%s'%(val,))

        # event_handlers.new(self, None)
        self.model.saved=True
        #### End Manual config
        debug('finished setup of MainFrame')

    def ScriptEditorKeyEvent(self, evt):
        if evt.GetKeyCode()==13:
            pos=self.script_editor.GetCurrentPos()
            line=self.script_editor.GetCurrentLine()
            idn=self.script_editor.GetLineIndentation(line)
            txt=self.script_editor.GetLine(line).strip()
            if txt.startswith('for ') or txt.startswith('if ') or \
                    txt.startswith('elif ') or txt.startswith('else:'):
                idn+=4
            self.script_editor.InsertText(pos, '\n'+' '*idn)
            self.script_editor.GotoPos(pos+idn+1)
        else:
            evt.Skip()

    def __set_properties(self):
        # self.main_frame_toolbar.SetToolBitmapSize((32,32))
        # self.main_frame_toolbar.SetBackgroundColour(wx.SystemSettings.GetColour(wx.SYS_COLOUR_MENUBAR))
        self.main_frame_fom_text=wx.StaticText(self.main_frame_toolbar, -1,
                                               '        FOM:                    ', size=(400, -1))
        font=wx.Font(wx.FontInfo(15*self.dpi_scale_factor))
        self.main_frame_fom_text.SetFont(font)
        self.main_frame_fom_text.SetLabel('        FOM: None')
        # self.main_frame_fom_text.SetEditable(False)
        self.main_frame_toolbar.AddSeparator()
        self.main_frame_toolbar.AddSeparator()
        self.main_frame_toolbar.AddControl(self.main_frame_fom_text)

        # begin wxGlade: MainFrame.__set_properties
        self.SetTitle("GenX")
        _icon=wx.NullIcon
        _icon.CopyFromBitmap(img.genx.GetBitmap())
        self.SetIcon(_icon)
        self.main_frame_statusbar.SetStatusWidths([-2, -3, -2])

        # statusbar fields
        main_frame_statusbar_fields=["", "", "x,y"]
        for i in range(len(main_frame_statusbar_fields)):
            self.main_frame_statusbar.SetStatusText(main_frame_statusbar_fields[i], i)
        self.main_frame_toolbar.Realize()
        self.data_grid_choice.SetSelection(0)
        self.static_line_1.SetMinSize((-1, 5))
        self.data_grid.CreateGrid(10, 6)
        self.data_grid.EnableEditing(0)
        self.data_grid.EnableDragRowSize(0)
        self.data_grid.SetColLabelValue(0, "x_raw")
        self.data_grid.SetColLabelValue(1, "y_raw")
        self.data_grid.SetColLabelValue(2, "Error_raw")
        self.data_grid.SetColLabelValue(3, "x")
        self.data_grid.SetColLabelValue(4, "y")
        self.data_grid.SetColLabelValue(5, "Error")
        self.plot_splitter.SetMinimumPaneSize(20)
        self.hor_splitter.SetMinimumPaneSize(20)
        self.ver_splitter.SetMinimumPaneSize(20)
        # end wxGlade
        # Turn Line numbering on for the editor
        self.script_editor.setDisplayLineNumbers(True)
        self.ver_splitter.SetMinimumPaneSize(1)
        self.hor_splitter.SetMinimumPaneSize(1)

    def __do_layout(self):
        # begin wxGlade: MainFrame.__do_layout
        frame_sizer=wx.BoxSizer(wx.VERTICAL)
        main_sizer=wx.BoxSizer(wx.HORIZONTAL)
        input_sizer=wx.BoxSizer(wx.VERTICAL)
        sizer_8=wx.BoxSizer(wx.HORIZONTAL)
        sizer_7=wx.BoxSizer(wx.HORIZONTAL)
        plot_sizer=wx.BoxSizer(wx.HORIZONTAL)
        sizer_6=wx.BoxSizer(wx.HORIZONTAL)
        sizer_5=wx.BoxSizer(wx.HORIZONTAL)
        sizer_4=wx.BoxSizer(wx.HORIZONTAL)
        sizer_3=wx.BoxSizer(wx.HORIZONTAL)
        data_sizer=wx.BoxSizer(wx.VERTICAL)
        sizer_1=wx.BoxSizer(wx.VERTICAL)
        sizer_2=wx.BoxSizer(wx.HORIZONTAL)
        data_list_sizer=wx.BoxSizer(wx.HORIZONTAL)
        data_list_sizer.Add(self.data_list, 1, wx.EXPAND, 0)
        self.data_notebook_data.SetSizer(data_list_sizer)
        sizer_1.Add((20, 5), 0, 0, 0)
        sizer_2.Add(self.label_2, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_2.Add(self.data_grid_choice, 3, wx.EXPAND, 0)
        sizer_2.Add((20, 20), 0, wx.EXPAND, 0)
        sizer_1.Add(sizer_2, 0, wx.EXPAND, 0)
        sizer_1.Add(self.static_line_1, 0, wx.EXPAND, 0)
        sizer_1.Add(self.data_grid, 1, wx.EXPAND, 0)
        self.data_notebook_pane_2.SetSizer(sizer_1)
        self.data_notebook.AddPage(self.data_notebook_data, "Data")
        self.data_notebook.AddPage(self.data_notebook_pane_2, "View")
        data_sizer.Add(self.data_notebook, 1, wx.EXPAND, 0)
        self.data_panel.SetSizer(data_sizer)
        sizer_3.Add(self.plot_data, 2, wx.EXPAND, 0)
        self.plot_notebook_data.SetSizer(sizer_3)
        sizer_4.Add(self.plot_fom, 1, wx.EXPAND, 0)
        self.plot_notebook_fom.SetSizer(sizer_4)
        sizer_5.Add(self.plot_pars, 1, wx.EXPAND, 0)
        self.plot_notebook_Pars.SetSizer(sizer_5)
        sizer_6.Add(self.plot_fomscan, 1, wx.EXPAND, 0)
        self.plot_notebook_foms.SetSizer(sizer_6)
        self.plot_notebook.AddPage(self.plot_notebook_data, "Data")
        self.plot_notebook.AddPage(self.plot_notebook_fom, "FOM")
        self.plot_notebook.AddPage(self.plot_notebook_Pars, "Pars")
        self.plot_notebook.AddPage(self.plot_notebook_foms, "FOM scans")
        self.wide_plugin_notebook.AddPage(self.panel_1, "Empty Tab")
        self.plot_splitter.SplitVertically(self.plot_notebook, self.wide_plugin_notebook)
        plot_sizer.Add(self.plot_splitter, 1, wx.EXPAND, 0)
        self.plot_panel.SetSizer(plot_sizer)
        sizer_7.Add(self.paramter_grid, 1, wx.EXPAND, 0)
        self.input_notebook_grid.SetSizer(sizer_7)
        sizer_8.Add(self.script_editor, 1, wx.EXPAND, 0)
        self.input_notebook_script.SetSizer(sizer_8)
        self.input_notebook.AddPage(self.input_notebook_grid, "Grid")
        self.input_notebook.AddPage(self.input_notebook_script, "Script")
        input_sizer.Add(self.input_notebook, 1, wx.EXPAND, 0)
        self.input_panel.SetSizer(input_sizer)
        self.hor_splitter.SplitHorizontally(self.plot_panel, self.input_panel)
        main_sizer.Add(self.hor_splitter, 1, wx.EXPAND, 0)
        self.main_panel.SetSizer(main_sizer)
        self.ver_splitter.SplitVertically(self.data_panel, self.main_panel)
        frame_sizer.Add(self.ver_splitter, 1, wx.EXPAND, 0)
        self.SetSizer(frame_sizer)
        frame_sizer.Fit(self)
        frame_sizer.SetSizeHints(self)
        self.Layout()
        self.Centre()
        # end wxGlade
        self.sep_plot_notebook=self.plot_notebook
        if self.config.get_boolean('startup', 'widescreen', False):
            # test adding new notebooks for plugins in wide screen layout
            self.plot_notebook=self.wide_plugin_notebook
            self.plot_notebook.RemovePage(0)
            self.plot_splitter.SetSashGravity(0.75)
            # self.plot_notebook = wx.Notebook(self.plot_panel, wx.ID_ANY,
            #                                 style=wx.NB_BOTTOM|wx.BORDER_SUNKEN)
            # plot_sizer.Add(self.plot_notebook, 1, wx.EXPAND|wx.ALL, 4)
            self.sep_data_notebook=self.data_notebook
            self.data_notebook=wx.Notebook(self.data_panel, wx.ID_ANY,
                                           style=wx.NB_TOP | wx.BORDER_SUNKEN)
            data_sizer.Add(self.data_notebook, 1, wx.EXPAND | wx.ALL, 4)
        else:
            self.plot_splitter.Unsplit()

    def Show(self):
        ''' Overiding the default method since any resizing has to come AFTER
            the calls to Show
        '''
        display_size=wx.DisplaySize()
        hsize=self.config.get_int('gui', 'hsize', int(display_size[0]*0.85))
        vsize=self.config.get_int('gui', 'vsize', int(display_size[1]*0.9))
        self.SetSize(hsize, vsize)
        self.CenterOnScreen()
        self.ver_splitter.SetSashPosition(200)
        self.hor_splitter.SetSashPosition(200)
        # Gravity sets how much the upper/left window is resized default 0
        self.ver_splitter.SetSashGravity(0.25)
        self.hor_splitter.SetSashGravity(0.75)

        wx.Frame.Show(self)
        ## Begin Manual Config
        wx.CallAfter(self.LayoutSplitters)

    def LayoutSplitters(self):
        size=self.GetSize()
        vsplit=self.config.get_int('gui', 'vsplit', size[0]/4)
        hsplit=self.config.get_int('gui', 'hsplit', size[1]-450)
        self.ver_splitter.SetSashPosition(vsplit)
        self.hor_splitter.SetSashPosition(hsplit)

        if self.config.get_boolean('startup', 'widescreen', False):
            psplit=self.config.get_int('gui', 'psplit', int(size[1]*0.6))
            self.plot_splitter.SetSashPosition(psplit)

    def startup_dialog(self, profile_path, force_show=False):
        show_profiles=self.config.get_boolean('startup', 'show profiles')
        widescreen=self.config.get_boolean('startup', 'widescreen', False)
        if show_profiles or force_show:
            startup_dialog=StartUpConfigDialog(self, profile_path+'profiles/',
                                               show_cb=show_profiles,
                                               wide=widescreen)
            startup_dialog.ShowModal()
            config_file=startup_dialog.GetConfigFile()
            if config_file:
                self.config.load_default(profile_path+'profiles/'+config_file, reset=True)
                self.config.default_set('startup', 'show profiles',
                                        startup_dialog.GetShowAtStartup())
                self.config.default_set('startup', 'widescreen',
                                        startup_dialog.GetWidescreen())
                self.config.write_default(os.path.join(config_path, 'genx.conf'))
                debug('Changed profile, plugins loaded=%s'%self.config.get('plugins', 'loaded plugins'))
                try:
                    self.plugin_control.OnOpenModel(None)
                except Exception as e:
                    outp=io.StringIO()
                    traceback.print_exc(200, outp)
                    val=outp.getvalue()
                    outp.close()
                    error("Exception:\n%s"%outp)
                    wx.ShowErrorDialog(self, 'Problems when plugins processed model.' \
                                             ' Python Error:\n%s'%(val,))

    def eh_mb_new(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.new(self, event)

    def eh_mb_open(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.open(self, event)

    def eh_mb_save(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.save(self, event)

    def eh_mb_print_plot(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.print_plot(self, event)

    def eh_mb_print_grid(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.print_parameter_grid(self, event)

    def eh_mb_print_script(self, event):  # wxGlade: MainFrame.<event_handler>
        warning("Event handler `eh_mb_print_script' not implemented")
        event.Skip()

    def eh_mb_export_orso(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.export_orso(self, event)

    def eh_mb_export_data(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.export_data(self, event)

    def eh_mb_export_table(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.export_table(self, event)

    def eh_mb_export_script(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.export_script(self, event)

    def eh_mb_quit(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.quit(self, event)

    def eh_mb_copy_graph(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.copy_graph(self, event)

    def eh_mb_copy_sim(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.copy_sim(self, event)

    def eh_mb_copy_table(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.copy_table(self, event)

    def eh_mb_view_zoom(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.on_zoom_check(self, event)
        event.Skip()

    def eh_mb_view_grid_slider(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.on_grid_slider_check(self, event)
        event.Skip()

    def eh_mb_fit_start(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.start_fit(self, event)

    def eh_mb_fit_stop(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.stop_fit(self, event)

    def eh_mb_fit_resume(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.resume_fit(self, event)

    def eh_mb_fit_analyze(self, event):  # wxGlade: MainFrame.<event_handler>
        warning("Event handler `eh_mb_fit_analyze' not implemented")
        event.Skip()

    def eh_mb_misc_showman(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.show_manual(self, event)

    def eh_mb_misc_about(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.show_about_box(self, event)

    def eh_data_grid_choice(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.change_data_grid_view(self, event)

    def eh_tb_new(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.new(self, event)

    def eh_tb_open(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.open(self, event)

    def eh_tb_save(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.save(self, event)

    def eh_tb_simulate(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.simulate(self, event)

    def eh_tb_start_fit(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.start_fit(self, event)

    def eh_tb_stop_fit(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.stop_fit(self, event)

    def eh_tb_restart_fit(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.resume_fit(self, event)

    def eh_tb_zoom(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.on_zoom_check(self, event)

    def eh_new_model(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.on_new_model(self, event)
        event.Skip()

    def eh_mb_saveas(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.save_as(self, event)

    def eh_ex_status_text(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.status_text(self, event)

    def eh_ex_point_pick(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.point_pick(self, event)

    def eh_ex_plot_settings_changed(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.plot_settings_changed(self, event)
        event.Skip()

    def eh_tb_calc_error_bars(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.calculate_error_bars(self, event)

    def eh_tb_error_stats(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.error_stats(self, event)

    def eh_plot_page_changed(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.plot_page_changed(self, event)
        event.Skip()

    def eh_mb_view_zoomall(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.zoomall(self, event)
        event.Skip()

    def eh_mb_view_yscale_log(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.set_yscale(self, 'log')
        event.Skip()

    def eh_mb_view_yscale_linear(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.set_yscale(self, 'linear')
        event.Skip()

    def eh_mb_view_xscale_log(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.set_xscale(self, 'log')
        event.Skip()

    def eh_mb_view_xscale_linear(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.set_xscale(self, 'linear')
        event.Skip()

    def eh_mb_view_autoscale(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.on_autoscale(self, event)
        event.Skip()

    def eh_mb_use_cuda(self, event):
        if self.main_frame_menubar.mb_use_cuda.IsChecked():
            event_handlers.activate_cuda(self, event)
        else:
            event_handlers.deactivate_cuda(self, event)
        event.Skip()

    def eh_mb_set_opt(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.on_optimizer_settings(self, event)

    def eh_mb_import_data(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.import_data(self, event)

    def eh_mb_import_table(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.import_table(self, event)

    def eh_mb_import_script(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.import_script(self, event)

    def eh_external_fom_value(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.fom_value(self, event)
        event.Skip()

    def eh_mb_set_dal(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.on_data_loader_settings(self, event)

    def eh_external_update_data_grid_choice(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.update_data_grid_choice(self, event)
        event.Skip()

    def eh_external_update_data(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.update_data(self, event)
        event.Skip()

    def eh_mb_fit_evaluate(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.evaluate(self, event)

    def eh_data_new_set(self, event):  # wxGlade: MainFrame.<event_handler>
        self.data_list.eh_tb_add(event)

    def eh_data_new_simulation_set(self, event):  # wxGlade: MainFrame.<event_handler>
        self.data_list.eh_tb_add_simulation(event)

    def eh_data_delete(self, event):  # wxGlade: MainFrame.<event_handler>
        self.data_list.eh_tb_delete(event)

    def eh_data_move_down(self, event):  # wxGlade: MainFrame.<event_handler>
        self.data_list.list_ctrl.MoveItemDown()

    def eh_data_move_up(self, event):  # wxGlade: MainFrame.<event_handler>
        self.data_list.list_ctrl.MoveItemUp()

    def eh_data_toggle_show(self, event):  # wxGlade: MainFrame.<event_handler>
        self.data_list.list_ctrl.OnShowData(event)

    def eh_data_toggle_use(self, event):  # wxGlade: MainFrame.<event_handler>
        self.data_list.list_ctrl.OnUseData(event)

    def eh_data_toggle_error(self, event):  # wxGlade: MainFrame.<event_handler>
        self.data_list.list_ctrl.OnUseError(event)

    def eh_data_calc(self, event):  # wxGlade: MainFrame.<event_handler>
        self.data_list.list_ctrl.OnCalcEdit(event)

    def eh_data_import(self, event):  # wxGlade: MainFrame.<event_handler>
        self.data_list.list_ctrl.OnImportSettings(event)

    def eh_data_plots(self, event):  # wxGlade: MainFrame.<event_handler>
        self.data_list.list_ctrl.OnPlotSettings(event)

    def eh_mb_models_help(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.models_help(self, event)

    def eh_external_model_changed(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.models_changed(self, event)
        event.Skip()

    def eh_mb_plugins_help(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.plugins_help(self, event)

    def eh_mb_data_loaders_help(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.data_loaders_help(self, event)

    def eh_mb_findreplace(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.on_findreplace(self, event)

    def eh_external_find(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.on_find_event(self, event)

    def eh_mb_fom_help(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.fom_help(self, event)

    def eh_mb_view_use_toggle_show(self, event):  # wxGlade: MainFrame.<event_handler>
        new_val=self.main_frame_menubar.mb_use_toggle_show.IsChecked()
        self.data_list.list_ctrl.SetShowToggle(new_val)

    def eh_mb_misc_openhomepage(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.show_homepage(self, event)

    def eh_show_startup_dialog(self, event):  # wxGlade: MainFrame.<event_handler>
        self.startup_dialog(config_path, force_show=True)

    def eh_external_parameter_value_changed(self, event):  # wxGlade: MainFrame.<event_handler>
        event_handlers.parameter_value_changed(self, event)

    def eh_mb_fit_autosim(self, event):  # wxGlade: MainFrame.<event_handler>
        warning("Event handler 'eh_mb_fit_autosim' not implemented!")
        event.Skip()

# end of class MainFrame


class MyApp(wx.App):
    def __init__(self, show_startup, *args, **kwargs):
        debug('App init started')
        self.show_startup=show_startup
        wx.App.__init__(self, *args, **kwargs)
        debug('App init complete')

    def OnInit(self):
        debug('entering init phase')
        locale=wx.Locale(wx.LANGUAGE_ENGLISH)
        self.locale=locale

        main_frame=MainFrame(self, None, -1, "")
        self.SetTopWindow(main_frame)

        # main_frame.Show()
        if self.show_startup:
            main_frame.startup_dialog(config_path)

        debug('init complete')
        wx.CallAfter(main_frame.plugin_control.LoadDefaultPlugins)
        wx.CallAfter(main_frame.Show)
        return 1

# end of class MyApp

class StartUpConfigDialog(wx.Dialog):
    def __init__(self, parent, config_folder, show_cb=True, wide=False):
        wx.Dialog.__init__(self, parent, -1, 'Change Startup Configuration')

        self.config_folder=config_folder
        self.selected_config=None

        sizer=wx.BoxSizer(wx.VERTICAL)
        sizer.Add((-1, 10), 0, wx.EXPAND)

        sizer.Add(wx.StaticText(self, label='Choose the profile you want GenX to use:            '),
                  0, wx.ALIGN_LEFT, 5)
        self.profiles=self.get_possible_configs()
        self.config_list=wx.ListBox(self, size=(-1, 200), choices=self.profiles, style=wx.LB_SINGLE)
        self.config_list.SetSelection(self.profiles.index('SimpleReflectivity'))
        sizer.Add(self.config_list, 1, wx.GROW | wx.TOP, 5)

        startup_cb=wx.CheckBox(self, -1, "Show at startup", style=wx.ALIGN_LEFT)
        startup_cb.SetValue(show_cb)
        self.startup_cb=startup_cb
        sizer.Add((-1, 4), 0, wx.EXPAND)
        sizer.Add(startup_cb, 0, wx.EXPAND, 5)
        wide_cb=wx.CheckBox(self, -1, "Widescreen (need restart)", style=wx.ALIGN_LEFT)
        wide_cb.SetValue(wide)
        self.wide_cb=wide_cb
        sizer.Add(wide_cb, 0, wx.EXPAND, 5)

        sizer.Add((-1, 4), 0, wx.EXPAND)
        sizer.Add(wx.StaticText(self, label='These settings can be changed at the menu:\n Options/Startup Profile'),
                  0, wx.ALIGN_LEFT, 5)

        # Add the Dilaog buttons
        button_sizer=wx.StdDialogButtonSizer()
        okay_button=wx.Button(self, wx.ID_OK)
        okay_button.SetDefault()
        button_sizer.AddButton(okay_button)
        button_sizer.AddButton(wx.Button(self, wx.ID_CANCEL))
        button_sizer.Realize()
        # Add some eventhandlers
        self.Bind(wx.EVT_BUTTON, self.OnClickOkay, okay_button)

        line=wx.StaticLine(self, -1, size=(20, -1), style=wx.LI_HORIZONTAL)
        sizer.Add(line, 0, wx.GROW | wx.TOP, 20)

        sizer.Add((-1, 4), 0, wx.EXPAND)
        sizer.Add(button_sizer, 0,
                  flag=wx.ALIGN_RIGHT, border=20)
        sizer.Add((-1, 4), 0, wx.EXPAND)

        main_sizer=wx.BoxSizer(wx.HORIZONTAL)
        main_sizer.Add((10, -1), 0, wx.EXPAND)
        main_sizer.Add(sizer, 1, wx.EXPAND)
        main_sizer.Add((10, -1), 0, wx.EXPAND)
        self.SetSizer(main_sizer)

        sizer.Fit(self)
        self.Layout()
        self.CentreOnScreen()

    def OnClickOkay(self, event):
        self.selected_config=self.profiles[self.config_list.GetSelection()]
        self.show_at_startup=self.startup_cb.GetValue()
        self.widescreen=self.wide_cb.GetValue()
        event.Skip()

    def GetConfigFile(self):
        if self.selected_config:
            return self.selected_config+'.conf'
        else:
            return None

    def GetShowAtStartup(self):
        return self.show_at_startup

    def GetWidescreen(self):
        return self.widescreen

    def get_possible_configs(self):
        '''get_possible_configs(self) --> list of strings
        
        search the plugin directory. 
        Checks the list for python scripts and returns a list of 
        module names that are loadable .
        '''
        # Locate all python files in this files directory
        # but excluding this file and not loaded.
        plugins=[s[:-5] for s in os.listdir(self.config_folder) if '.conf'==s[-5:]
                 and s[:2]!='__']
        return plugins

if __name__=="__main__":
    app=MyApp(True, 0)
    app.MainLoop()



# ----------------------------------------------------------------------
# This file was generated by exportimages.py
#
from wx.lib.embeddedimage import PyEmbeddedImage

copy=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAHeSURBVDiN1ZXPaxNBFMc/M7OtKBUSxMZeJEgx1B9VKAqpP+hFEEQv+g941fbgzaM3PSgI"
    "RfHmxYugaMGDB6EqFdmDGMUGqqS0h6CNtqZBk2x2Z6aHNpK0u1upvfiF4Q3vvfnMvPnBCNq1"
    "BUgSrzJQjwqKlv5JILvKFyUXGA8LOCs2I6WTPTtyS6QPZHEcJywXExhm8i5jo1eOmKAxB+RX"
    "58hlo3ozR0/Re/hEJAxAOpI9/Vn2D56RKLU3coVKqa3buhJtpRZyr1Jvntw7XV380WOsla0x"
    "7fs4qlPj2DsYOxX43jXgdWvJbSoWPiSe3b16OX3wmDo0dEEiIrc18W16ctfkxNNxa+0g4IYC"
    "3z1/MJDsSYtzwzfjYAD0D52X9WpFT+dejhitXRmW9OtnaUf37oyzHqypVLpPKdXZB38OZa2E"
    "iAxFDokFblT/D9AazOYBtW7Mfy181JvBbK4wV5qdso9vD5v3Lx5SrSxsGNi82GVjzP2ZvHu8"
    "+CW3M/A871+BAPNoPebrKsC+3+XSANDxN5BaZQFjzByAisipL34vXuxKdovtyRQ68Al8b03z"
    "GzVmP71l4tGorwPvBpCLm/iSlKoG2LgmhAiQ8jorL2W9x9oBZFj+GsIUAJ+BWtOxBOMItoBE"
    "Vo2oAAAAAElFTkSuQmCC")
getcopyData=copy.GetData
getcopyImage=copy.GetImage
getcopyBitmap=copy.GetBitmap

# ----------------------------------------------------------------------
cut=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAK+SURBVDiNpdNbSBRRGAfw/8zuuLtnZlfHOd7wNkKFF0hdL0VtVhD00GtUJEVIdNEXiXqK"
    "kKAgS4IKC4Io6KnwJSgEKyqwh9TAS4RKaRJu7uw66+7s5qjrnh4yolB31f/b+c45P86Fj8P6"
    "UkgI2W+xWFINw7ixTuN3FEVpqnRXeo8dr49TqvT+P8+v0RNEkTQfPnIwZ2rKF4xEouc3BLpc"
    "rsY9e3fnxWIxDA1+GjdN891GQN7plM6UlBbbe3r6jGg0emUZi1rXcLr6nZ4d+Ywx9H7omzAM"
    "4xkACIJQK8tys82WUhOJRMeSBolELlRUlouDA0OzhhHqyMig9x0Ou6e4pDirqsot63ow/vRJ"
    "x8OkQELIgSq3W+V5Ht3d7/mq6uqmmtrqDEoVAMDkpBcvnneOB4PBu9zSnuzMzIz2WGzRY7Va"
    "GcdhyOfT6gHY0tLTTtlTbOcaTp4gXu8PhGZC2La9FppPQ29Pn398fEIzTbMzEAi0AfBxANIU"
    "qvRzAAG4xwyMZGdnHQr4A5aCgvxoYZGaY4TDXCgUhhE2QDOp/vXL2NScOfvW75++A2D4n+vI"
    "inxRLVLDAMr+1PLzcz+qqjp37fpVVl6xlbVcvsTONp1miqIMAahY7Xl4jnEWSRJ5SZLqAKiy"
    "LLcuLsZL5+fncPtW+4TD4TAdDju6ul59m56e9gDoXxXUdf2m5tNGRElskyRpOB5fbDRNs0/T"
    "/AdiCwufy8pK7aMjoyw8E34NIJTMJwIAB8ANoA6AvFQrk9PlhYLCgtjmLZtilNJ7yWLLhlL6"
    "klKqE4k0Uqq8ycvL1QC41u1JkvjT6XQ2LI33paa6ZgEUJtq4Ui+rKSk2GIYxAACKohzleT4C"
    "4HsicKVOGSMiiQiC8CjO4l4w5mGMbwUQTwRaVqjPCoJgiiLZZbVY6Pz8wgNd11sSYcmGS7zk"
    "b34Bv2kANxdpNYwAAAAASUVORK5CYII=")
getcutData=cut.GetData
getcutImage=cut.GetImage
getcutBitmap=cut.GetBitmap

# ----------------------------------------------------------------------
domain=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAJtSURBVDiNlZM/aFNBHMc/r7m2L0mb1CZp0n/YSsB2KbRLQaSDDi7dBFcVxN1BQZwEwUG6"
    "iFARhFIUHF066NDWqUqMotGhljb9E2rT/Gnz0qRJvPfOIYqU5j3sF95w9z587ne/u9OoJwD4"
    "sc8WoIBmB6YCKG2ok0vXRpkZCtLWiFIKnn/WEp88k3faI8O2wp8rCwfGWiwm2txM3Bgj4hb2"
    "S8+va30bAxO67u+xZYrZlVaDmNYkJZlCxWEjgImqmrWicoYsC1Aa0N17+uy8q7nVa8tWiu+C"
    "Wu6FW9dtmVK1upHYLMSFNzBwNTRxa0joDVsIQGpp9tz0eHJmJGQYdsz9RfTEJgjLlMvF7e8H"
    "ntBgY6NS1Mr7xuIasqOl8SlLYMtAAoiuWmr94t6TbK/Jod3qS5qqTG2OuZ4Wgrods2/kihDX"
    "RMjP5QcX1IDX4YY905vEj+p4JBA9v2vHtKy/141UXG9SGpZyPj8sC2VZliMlpQRQImnw6soc"
    "14XAbQcXpFY6LM+1pXMx8+gf158PqmUjB1TE4BluPp6it68bzU748YsZeP12N/jwnicpXJ2g"
    "hxEiAK5OXCIMIsDsy2/W7bs7PuF1E3GS/UsNKdMAiApIHQRg/qlxdMTn8vuF04NruHnqrTou"
    "NaUJpfr4hGks5e/cyYWNpZgApaPCbB6209Duha4geD3OUmS9MFEBKet3XkgT8WiarfhXEjsZ"
    "PhRLLLcIfB4Pw11Bov3dRPt7iFryuFJSILkGbxb2arGEVS3I1kMt1MFoZp8UkLEpxRUOM+nz"
    "sTcyTPmUG2FqqOwev/J5qjt5jNVV0kDtJE377/wGaRcACN5CN3MAAAAASUVORK5CYII=")
getdomainData=domain.GetData
getdomainImage=domain.GetImage
getdomainBitmap=domain.GetBitmap

# ----------------------------------------------------------------------
insert_domain=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAMrSURBVDiNfZRNaFxVGIafO/dM5tx786PNf9JghIjJJlhbELrIwi7cdCeoddG04latCApK"
    "oQUVMYhChWpBKSLWlQvFhQtTs6plTIOOXdTYJs1Mf2L+7/zfufd8LqYOScydb/UuXp7vvN/5"
    "zrGoVyfQQXxlAQGSTTwVQKzRfTxz4gAXR7to3cslAl/MWZlr7tE32/rGYoH35i8X/FvptGp1"
    "mHj5SfocFd96etHaf3t4QuuOgVhPfnU+5ZO2EmHIylalSRAgQqpRkJfmJmMAsYD+wUcen7aT"
    "KS/WW8nPdFlrXztax3qK1ertzNLWrPI6hye7J14fVXrPEQKQu/LV4fNPLVwc7/b9OM/ZX9CZ"
    "JVAmCm/k714vuN2P7k0UISht+hfm7I47fYNv6DadtG2FqYVmrHz/0uSQfz0Esj4hgOoJcotH"
    "Nj5dHYwox3W/YkllRgaOnz535HBbexuOdnG1y5kXvrMvXe05C7Dpr+Vh1lLdHTz77tMy7DXZ"
    "sAs6of7edKvVsESiZGGMQYwh5XlW78HJCkDL4q/az81qJRZGmt8fxiAioRTLBYwIIgYRQxiG"
    "kKp7wjAEELXg8+1zP3JSKZztkCAisaqVp9uTSCKpdWthpFguIA+ARgztnVG/tT71nrIV+uEg"
    "sJ8YeMs6OM4nX37Eq/v7sbYDT39mcWfcodVzcbTXmNsO7dS1oz1c7fLhye9/Vp5D324YQM2A"
    "ELI7phHBbNfGIEYQY7CTpJo8uP/mZyiVCo2Yu+F1XW8SRiGxwA4HskVo8UAQSuXijtOY+k01"
    "wGEtxF8rlGKBp44JH39TY6MAa5uQ/pPlQ6folUZkYeZcLutK4i9LtSDVRHVpbuOdHcDVdbi7"
    "DG0e9HTB2y/V9+m33+G1BSsL9FaDSiNm0q7988NU/wfK3kdmXvPi8fQ9FUaoqfNkZ/8gc3+F"
    "q/kiN1oU7a7LWE8XI0P9jAwNMMK2XQ1qAUaEKIqYv7XMT5c3gnTGVLfCVNnqfogDK5vkgJWY"
    "9HZvL0cjL/H8oVfUsZYH/40YuPZ5NK396MTNmywDAfD/dWlSbudjifftJF0AJqJSXDFnyuvk"
    "tpv+BWqctPtB+yPkAAAAAElFTkSuQmCC")
getinsert_domainData=insert_domain.GetData
getinsert_domainImage=insert_domain.GetImage
getinsert_domainBitmap=insert_domain.GetBitmap

# ----------------------------------------------------------------------
insert_layer=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAJ6SURBVDiN5ZO9axRBGMZ/szubndnL5e6SMzEBg1FREBEiIYUKET/AwkKIhY2F2PgHmFYQ"
    "G8H/wMIP0MJCFESCFoqdWqiFRvEDQTw1iURzd3t3u7c7YxHjB7mLwdanmedlZn6888wM/HcS"
    "gN7Yw6Eun1zrJQ44HrgeOIv+x4iH40nAZa4azrx5V7omd67hytl9HMz6iNZAg1AKofKgCwid"
    "Q6gCQucROg8qj9AFpkOTnDhzYZfs76Rva1872GKTISLwmapKpj43EX4DPxOyZ1sXXd0diECx"
    "brBHrspnhpwVBWMSnpUqnPO6qR4bJTk2Qv3wZk7efoSNKtiojI3KmDRBzsc0EgPyL+gXcwmD"
    "4/1ke120lmgV4Bc1RGVwPWpNS7VSrcgnsxzfe5nTjkAtB5x3GBw/Eg6H9SrGWqwxvJ2Zr4xN"
    "XL2L45Hilp+//zohZ8u8ni1zuBWko5PNQcHd6OKSJmJ3vVEbDutVrLVYa0hE+umDtRfdNKWe"
    "NJIwDGttLyPIMbJhvz+5fnu2qHyN8jV9gzmCIEOgM2gVYGMPU3fwfY1jPS6dundftgPKLme0"
    "b9gURbaOqwRSudSiEARYazHGEKgMmVUBWrsEStM/VFjdFrioJGnyMzdrFmDWLPXG0kxj2gJt"
    "KmJrFnyaptRqVawxv4EXvPnNx3FslnvQ3QPbxE1dFL0LpZCbDrhrs/kMgQrQKmBmynwrPW18"
    "kVIipUw/vQpvLXfkuY+P7Q6wAGQH3KNrx+x5qcIfHRnePGyUnl7/uuWP7P+WYSs1ojrWGuLE"
    "LJlbMdCkaWTTX98piiOSyC4hrhgYTjP58kbyQBdFEQBBOv+eOyvd/8/6DhXMJVdzbPBEAAAA"
    "AElFTkSuQmCC")
getinsert_layerData=insert_layer.GetData
getinsert_layerImage=insert_layer.GetImage
getinsert_layerBitmap=insert_layer.GetBitmap

# ----------------------------------------------------------------------
isometric=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAHSSURBVDiNxZJNSBRhGICfmZ2Zdnc2TW1TdNcVVyxi1SASEhLCQ2sHkegSHTsYSJcu4imI"
    "biF06FIWQeKhhOjgISqyqINgGBQqGqJFu+lSoCuU6zc7XydJd2fWxaKe4/vzfO/78cK/ptTc"
    "c/1gyMw2hkrXgI5dizSNeH1VYKXvbJ39Y+SETA21yQvxSLYhVLYIhN36FIdYpPaA+exInT96"
    "q7dBDZbo25IziXUuDS5Yc4nMq+S39Glgo6Bwv18VzfV7tdErTRia6rrB/ZffZf+9uexyWsSA"
    "2c14XkelqYjOijVaLo5z7eFnR9nA4yQ3H80rjeV4gNqtOccROsI2w6csvs4kiPWM83pqFYAX"
    "71c4fnmS5fkvPD+3TmtYl7m9mttKmgo9McGZKFy9/YGU0GmuUhnp+knQ79blMuFWgj4YaLfx"
    "qZI78cKyooSbOJ3DHwmL5f8JhWX9XaGu6TsXOQmldPt/p3DeGeYLVzeU7IOPurTza7dh2TC9"
    "lFEBT0FhIm0F7k5rN84/NTJvU57cNABjnzycHDbExJIxCDwp/PRvApFy/5vWakOMdiOP1Xjl"
    "bC+y67BX1OzzvQNKihXlcjRS5k1VmoodDXqTQGy3olwOFVP0C/lriBUsUM38AAAAAElFTkSu"
    "QmCC")
getisometricData=isometric.GetData
getisometricImage=isometric.GetImage
getisometricBitmap=isometric.GetBitmap

# ----------------------------------------------------------------------
orbit=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAFpSURBVDiNvdS/S1ZxFMfxV08p9gQOLoLYDx6KcHQJE3XKSQiXVofAKRvERZpc/BNaXBpL"
    "UiFycfDHkjmJjUmGJRW4+AP8Uepjwz3WJXqeLtzwAwe+33vPeZ9zvt9zL/9ZFzP6teIRPmMr"
    "L7AWz7CLHqxis5JzIQOwDTuYwzU8qeacBdiFeUzjCHV5gSW8Rxkz+JEhpqrmcDNVQF8150up"
    "dS0e4gQXsICvqPf7Esp4gW58wTr2/wYs4B7a0RKBJYzF+71UTB3u4xYasILXYb80iAmMYgjF"
    "eN6EN38UMBHdCGAvJjGQrrAG3zGLpVQbp9H+mYq4kdofSEbqAFeknGsi0wNcxxq+YRv9GJGc"
    "bSMe43mAr+ITXmEc5bMKj/AyrEFyjk1hhUhyjMs4xDKm8FGVr6aS0mNzJ5JWVJbB3kBzrJsl"
    "45ILuIbbsW7Bh7zAt+iQXODd2OcCLkpaHQ7ouwwx/1Qnnkp+tOern3aQSPwCX+wtAAAAAElF"
    "TkSuQmCC")
getorbitData=orbit.GetData
getorbitImage=orbit.GetImage
getorbitBitmap=orbit.GetBitmap

# ----------------------------------------------------------------------
pan=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AADzSURBVDiNrdQxSgRBEAXQN66Ma2LkAQQRFzyAGBt4AsXYQPAw7l5DjEwVPIC5oAjiGdRg"
    "QVjWoHtlkHGmZPvDp6Hnd1V1/emiMAZB3RG28NYnXA0GPMYM90F9J4Z4z1wrEfAC88yzPnHk"
    "yjt4QoW9pUpr4CazFyulMv4VsJb6NA6cneBcMu0HVV4PcYJTbEg9e2no9vP60NjbxQifuM68"
    "W1Q4yFwkqMQxl/7RWdvHYb7G5Nd+myljqT31P5J3BmxFcZcjw+ESB9jENm67xJEKHyU3R5L7"
    "S6OWBsMH1vvEkbf8hSupPdM+cXTATvGM16C+HL4BUo8mSicT6tEAAAAASUVORK5CYII=")
getpanData=pan.GetData
getpanImage=pan.GetImage
getpanBitmap=pan.GetBitmap

# ----------------------------------------------------------------------
paste=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAHLSURBVDiN7dS9axRBHMbx78zu3N7paXJBfDltBC3EGFLoFYKFChpOfGnEwka0sLBV8jfY"
    "CxKMhYWgIJpCJSciIhq0UMOBhV5iZUIUL4JIztvZmbEwye0WI6dtfMpnfnyYZfitIJsykJeh"
    "PK0CdR5cgDfCaKNHbGJvAnPAAoBYPFVhQC0x7AeIooih6mEGBwe83ORknfGHNdrtNlIQW8cp"
    "YCxcPD+IY9/FIUlewZWnsKeym+MnjnpBlcvx7HGNS4ckzxsuNzHlLsdJB+yLFGZzifDFFLRi"
    "uDZynbt37nnBZnOelobmD8eWElhHH0CYHlqIYewtVI6dw2iNNYkX3FgOKJbnuP36EdWdernP"
    "gLPfHNZJKkfOIuUf3mPplrMfuTFxn59aLHcyPaAtSCm7wgBClQPA2E4nPbP/nP/gigMDCdZa"
    "nLW++UxM8nuTZErJgOVegXCOxpsnXYHvX43Ts1pRUJ0us3rFCKoD8ODqMPXtuygUS17s+9cZ"
    "vnya5sxeRzu18ktgbCzCAQd2wNZ18OFzHd3y367QC/39gg1rBS+nHRI0dH6wm6SgsX4Nwaq8"
    "cF19byoz8y5ILKPGckGk+m3ASaDnLz0HvANuAckv5NKXBBlyzwYAAAAASUVORK5CYII=")
getpasteData=paste.GetData
getpasteImage=paste.GetImage
getpasteBitmap=paste.GetBitmap

# ----------------------------------------------------------------------
paste_new=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AALISURBVDiNnZTNa1RXGMZ/58y9cyeZG8zErzKmiLQR29JgwQYqFKyKSoraTXHRnS66LrT2"
    "H+im0mVBrJTSFsFCUdEqphURUReCGgIqGM1Oq+RDM8ncmbn3nPftIjGTaZlGfTbn8L4Pv/Me"
    "zsMxtKoMFGxgPwtz4eegOdrK+MxnR8TJMeAxkACY+W4Y5Bhyno8Aoihi1+BONm7sb4sbHh7h"
    "/LkhGo0G1pCKsg84Fcz3t6F8+OUuSyGE7y/B+wOb2PvJ7rbAMJ/n8oUhvtphuTKq+Wv39dvU"
    "NYE9UYhfUyK4eh9qKRw98iMnfj/ZFjg5OUUtg8lZpbcEovQABItNSQqnbsHAngP4LEO8awt8"
    "rZwjLj/mtxt/MvhOtlBvAT56qohaBj7ej7X/8x7Pp3w0xs/X/qCemYWaXWzIBKy1LwQDCMI8"
    "AF6aNdvG+8oKlrY0pSo8uHmI1/si8mGe6ngFY0yL56WA0+MPWf/BDO9t6abYGZNMdXHmhzyQ"
    "vuKEItQaVWaqFUSFeu2/niWB9dmnjN7+lSByuKTBG+Uas9UKqoK4gLVvr8D0RmwouDhJpG9J"
    "4N3hnyhtPkFcjCl2dFHsjJmpTiMqFDtiDh7fQbGzi2JHXDj8xV/fLAn0kuB8NndNUUQFEWld"
    "5/c2JGwB5iyICCqCsXOJMmYuk977hcmawEUHiJA2GtICLHcbjCqjNy+yftN2AHrX7mT08j2w"
    "DvFCT/8YWhZUhTRxnP9ujJXLAqZrgauMu9MtwDiCwX44e/hrRvrepSMuzXe6AUjrCdFqpVCa"
    "RVQgyVj19wTLJ4QL1/WZ8/zyHJh6wSiw9S1YtwLuPRkh+1cskhQMAWCo1RMkcfT2QOUZWMig"
    "GZurqSM7dFaks2C03QOpQv6uFArLrQGYuFPTkWFpPJlS65WT0PyxAd4EPgWWtQMC2BzdQcQq"
    "AJ8x5TPGgdvAccD9A477Uz6v2B3/AAAAAElFTkSuQmCC")
getpaste_newData=paste_new.GetData
getpaste_newImage=paste_new.GetImage
getpaste_newBitmap=paste_new.GetBitmap

# ----------------------------------------------------------------------
sample=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAJmSURBVDiNlZO9TxNxHIefux70hUJb2hPaIgWNCBIGZDCR0AFNTFwcHBw00UTj7p9g4mCC"
    "8SWa6Kar/4KJii8JEeliFWFAS4tguYa2tA2lvd7PoVWw9Kp8lrtf7snzfbn8JKrxAi7MkwAc"
    "Na4ZU5YGOzlzZZRngz6cjSgh4OGcZfFrx1m1vXuo08ymLUxH07HZk4rTTvjacbrtinnpp/Ot"
    "vd6BsGpz+U2ZvLYUSMdmZVnX0bLFJoMALaJSrGwXys0YUdG3ASEB/mDo6CtLi7XNDNaLm+9D"
    "yvpQh11quEMJ0PK8iyQKF5U2b99lNXxjULE1XCEAP14/CE9NxPyj3chmzM03nIgkkBSjoi/m"
    "Vr/kHWp/Y6MQlMrF7PQyVrcNXyPEEBDLkgOEcqC0EjuVfpQKVtgyq/5WFoX78fGNJ7mgMGMK"
    "+W9bMGdRVBfnb02KvrYW04lZ27Lakr3n2m3ugCmzsTSjpJfnDspCwhCmdauREEIIoykjBAIw"
    "JK+ToZEeXlgU7GZwdlvejBsjVovNY63+09+Vdt71Qmpd+/5pTOk/xPV7dwj2BnaT9TG8kryE"
    "rPiQFbX29P11Xlmj88Klu7cVhx1/c1ltJCOPof851X+lx69KTmdrV5ML9y9pA7Eosy/hjnRv"
    "hwCGUdq/sCot1DqtF9d1mNqA1SRYW8HfBR3mt3GXdJdYlFD0CsrUYxKRKJ9/pviYzxLVwel1"
    "MexxM3AwwOHJcUKnJ3DabXulyaTg5Uw+8yESj+cKpYikuhnVMiSAlEkzksPB2LEjXA0FGfa4"
    "8BgGZLJoWpr1TJb56ALPgcX/Wde+8wsuf/VmF6kkGwAAAABJRU5ErkJggg==")
getsampleData=sample.GetData
getsampleImage=sample.GetImage
getsampleBitmap=sample.GetBitmap

# ----------------------------------------------------------------------
selection=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAG3SURBVDiN3dK/axphHAbw72tPT1POH9wZkxMCiuI/4IFK/oluRx2EAxdxcBJ1yCRYhFpw"
    "LK02QYVkEFyE4NC1o1gwkM2AJINgpsRrY55OmjYVPG2H0gfe4f2+8HmfF16i/zoA9sbj8XsA"
    "+38K7dZqtW4gEPiuqqoOgNsWEhuNxlkoFHoQBAFOpxPtdvvTVlCr1TpVFEV3OBwgIhARgsHg"
    "HIBnE0hoNpvHiqLooiguISICx3Eol8tfDUOdTudDLBbTJUn6BVosr9c7H41Gh+ugF91u92M0"
    "Gv3mdrtXQkQExhgSicTtKsP0bM/6/f4OgHuLxTLneX7lxR6PB8lk8o2h5y7S6/UONE275Hn+"
    "t4aRSEQHYDPScBlZlm8mk4lV13XiuKdv5nK5KJVK9Rhj94bbATCpqnpBRJBleZ7L5T6LovhI"
    "RPD5fA+z2cxvGCMiisfjX4gIkiQ9ViqV1wBM4XD4zmw2o1AojDbC0un0ORHBbrejVCqlF/Ns"
    "NvvWarViOBy+MoxlMpkTxhhsNhuKxeLRz2f1en2vWq1eA2CGMAAvB4PBld/vRz6ff2e4xRpU"
    "nk6n2l/B/tn8AOiszSoOz6k8AAAAAElFTkSuQmCC")
getselectionData=selection.GetData
getselectionImage=selection.GetImage
getselectionBitmap=selection.GetBitmap

# ----------------------------------------------------------------------
x=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAKwSURBVDiNzZJdSFNxGMZ/+3adOadzc37VtPzApIuQlkISoReZEAZdeGEXEWFkGl1IBkIQ"
    "QVh0VyEkGBLRTRiFJWZFgiQElg5Ss7T8nE2na+6jTU8X89SyjW59rs5539/z/p/znj9sdcmk"
    "B52a5lQdtsimy0+r08uTTZ787ERaFDLkUsHhZdDtp/kvSgklBcm4hmsR7WcQ359G3GFgFjBF"
    "YKpUHSM9NWFmuBax0MxKnJKDEvD7lBD0L/po7xrHm2WAHCPcOYwlXc8DiTELXKu3kVmSAVkG"
    "eDqGb9FLuz/Ea4lRRKZcDdJrX6C6PBuTSQCrAZl9gaQpN9NrIdT5JlpuH8EgAz46oaGb8XkP"
    "x4D1f3YYobwCE2/6T2JWycHzE2xtzAXWCLw6gTVTD74Q7G/DMb5ECfAl0qyIMnBRlLHu9FJc"
    "loVGrYDCFIS9FoQD28N8XReuIQcXfSFebjZHSwggS9HR136U4tLtf/YM0P2Z0Lln9Mz8oCKq"
    "McZAAIs1gcGBU1h06nDB5YOiu8zNe8gH3NFM8mjFDQkaFXJVxFIS4iBTj1oFebFMsQYqLTo6"
    "71dh1iggsAYiIJdBRxXG5HgeAtpoxmg/BYvA9XobZZW5qAEaX7Ay6UIsSkOVoIHEOLa9m8Xq"
    "CdL534RKKLYaqKm3IQAMzCA+HmPkxlvmpja2VrMHzW4zlXFKymJ9uiQhXc/k6FnE1SbEpUbE"
    "nUnMARkqsO1Lw+lpCve+nUdMj2caSIyZ0KLj3tVDpGbow+9NvbhX/FwGpoMw8NXNo44P+AGM"
    "WrhVgSUtno6oAxPUHM81Ul6UinpiGZ6PI3aOYnd6aZUYh4eGK304hhwwsQy7klDkGilN1FIt"
    "MdI9lBk03DQJ5EiN4DrByWXqgJnIBFolJRl6LkXWvq/yaTnABcKXYYvrF1mX2jQHt3dFAAAA"
    "AElFTkSuQmCC")
getxData=x.GetData
getxImage=x.GetImage
getxBitmap=x.GetBitmap

# ----------------------------------------------------------------------
y=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAH8SURBVDiNzZI/TBNhGMZ/17v0CrRH06QFcsbEBAeJDRubi8KAsmK0NakLBhNH/8QmRicT"
    "FxxMjH82Yoz/FgeigYijg06gQ4mcg1K4lOaAO2gpbc+hd7QpXkmc+izf+7zv+zzf+735oN0h"
    "OKcc7eRJOEDULRQrlH5vcgVYbxT0BLmg+Lnk8opNVTN4DHwEkJz8rujDHB8gkYjjB/jyh8rd"
    "zxSyFskGv6DsY+rdOH2iD4plOPeSPJByG3xusGZxa3oBPRKAY2FInERUFYaBAbcn1sW926eI"
    "9EdqPS8W2SrZ3AGMA4ZAYaPI9fR8vfholJga4plDo+EAyWQcGWDZgFffWTF2eNq4kkZDrBJv"
    "ZjUyC3qNx2MwpHJCkhjpC/Lw/mmiorP1qzOsr1mkgKqnIUDWJDU5Q852+INhIt1+nvcGGRnt"
    "RwR4n6GsGcwCX5v1YnMCyAsCRyMdDA72ICkybBaRrw3RrSoIO3tw/i36iskYUGwWC/8wBOhU"
    "Q2S+TXBEkaFcBcl5S3qerelFbjbvrtWEAHs2ZHWTM2eP0+Fzrl02IP2Jpdw2Ex66gzt0YZV4"
    "/UHj51K+nrsxRy5rcRmwvXSehgBVmznNqPNVkwrwo5WmpeH/oP0NpcMa9G34tVGLd6uev2If"
    "Xv8QgJDMWG8Xky63ShRWLS4C5UNHbVv8BdrTml0zMuGBAAAAAElFTkSuQmCC")
getyData=y.GetData
getyImage=y.GetImage
getyBitmap=y.GetBitmap

# ----------------------------------------------------------------------
z=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAHGSURBVDiNzZO9S1thFMZ/NzGmzU2GGvVWYoVQkUxCJJMVF8Fa7SKoLdSAgyL+AZVmVQod"
    "XbpkKw4itpY6KIqLNNAhgh1K26EfYGibaExNvDFqbnI73II339fNZzvPe85znsN7Dlx3CPrA"
    "ZWfeVo/XaHEkxcKZwraeq9MHqsDY6hgdgkBNbH6HuR12qgoCuG8V2a6AJlt5vkDwQuHQF6S+"
    "OElRMbscNKyMIooWjfsSJ5PNkzTQu7SpJLK78YRcOoCaDqBu+cm1OAgXGypxWA6SSPBpN57e"
    "NkwA+0mYeMuvPzIPAKU431RNrNHGZHcrwzM+RICMAiMrHPyWGQLiBifUYIGuTolYYlYbUw6g"
    "DraTcN7AX62ukkNJcrD2ZpRmq1kjXoSQ96IsHp2xeCVngKXFzset8ctPePeIrGTnA2CuVVzi"
    "ULLz6lkPHffuaG/fEjC9TiQmMwDkriTYaGO63839SS83AeQLGHnNYfSEh2Bs5/RH4Wxv4PPu"
    "FM2W/22eh8gtfeLAauK8XHHslOO/mcLb1wu29rkJrz3mthEnAJ6X/IikuKvnChY7k8X889io"
    "HKhlzl4vmN5P8n54GatRwbzKV+Ptrwv+AQc3h7C1xoWhAAAAAElFTkSuQmCC")
getzData=z.GetData
getzImage=z.GetImage
getzBitmap=z.GetBitmap

# ----------------------------------------------------------------------
zoom_small=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAInSURBVDiNrdMxaBNRGAfw/x13RwjhCsmB2NwJpxy5dkiPJpLQLEU0EJEki5McZOrgUGiG"
    "4CQUr0UcDJRktUuHrgGjaJA4ZReaIRyIEjKcQwOp0nDWfC5NKcpda+x/e+997/e9B+8BVxzm"
    "soVEFAVwE8AJgAOGYY5m6jgYDO5alvV5dXWVFEUhXdfJNM1fzWbzHRHd+Ces1Wo9WVxcJADE"
    "siyFw+HvwWDQBUAMw1C5XP7huu7ypbBut3tH13UCQPF4/AOA66dLrCRJDyRJGgIgy7KGRBS6"
    "EKxUKjYASiaTLY+SeVEUx6FQiHq93lNfjIjmDMOY8Dw/EUUx7FWnaVoVAFWr1d50jvWoVWzb"
    "ZhRFGY1Go0Mv0LbtBgD0+/35i8BjQRDgui7vd5NIJDIHABzHuReBX5aWln4OBoNgPp/PeIGy"
    "LD8GAE3TDvwaAwC2trZeAyBd1w9TqZT453oikXjIcdxEVVVyHCfri7Xbba5QKHwFQABIluXj"
    "QqHwPB6PL6fT6Vwmk3nD8/yE53na29t764sREWOa5qcphtNHfH4MgKLRKO3u7jaJSDi//6+/"
    "vL6+/n5nZ+fedJzL5b6VSqWXnU7nkeM4ciAQcGOxWLdYLD5bWFj46Hu6jY2NVyzLnp1iZWXl"
    "qNFoXPPd5JXt7e1NQRDOMMMwxvv7+7dmwgDAcZwX2WyWAJCqqif1ev32zNg0w+GwvLa2Nq7V"
    "avf/G5uGiKQrw64ivwHZFc3aocTgCAAAAABJRU5ErkJggg==")
getzoom_smallData=zoom_small.GetData
getzoom_smallImage=zoom_small.GetImage
getzoom_smallBitmap=zoom_small.GetBitmap



# -*- coding: utf8 -*-
'''<h1>Library for combined x-ray and neutrons simulations with adaptive layer segmentation.</h1>
<p>Most of the model is the same as in spec_nx.</p>
'''
from copy import deepcopy
from numpy import *
from scipy.special import wofz
from . import spec_nx
from .spec_nx import refl
from .lib.instrument import *

# Preamble to define the parameters needed for the models outlined below:
ModelID='SpecAdaptive'
__pars__=spec_nx.__pars__.copy()

instrument_string_choices=spec_nx.instrument_string_choices.copy()
InstrumentParameters=spec_nx.InstrumentParameters.copy()
InstrumentGroups=spec_nx.InstrumentGroups.copy()
InstrumentUnits=spec_nx.InstrumentUnits.copy()

LayerParameters=spec_nx.LayerParameters.copy()
LayerParameters.update({'magn_void': False, 'rough_type': 0, 'sigma_mag': 0.0})
LayerUnits=spec_nx.LayerUnits.copy()
LayerUnits.update({'magn_void': 'True/False',
                   'rough_type': '0-gauss/1-lin\n2/3-exp', 'sigma_mag': 'AA'})
LayerGroups=[('Standard', ['f', 'dens', 'd', 'sigma']),
             ('Neutron', ['b', 'xs_ai', 'magn', 'magn_ang', 'sigma_mag']),
             ('Special', ['magn_void', 'rough_type'])]

StackParameters={'Layers': [], 'Repetitions': 1, 'Element': 0}
SampleParameters=spec_nx.SampleParameters.copy()
SampleParameters.update({'minimal_steps': 0.5, 'max_diff_n': 0.01,
                         'max_diff_x': 0.01, 'smoothen': 0})
sample_string_choices={}

AA_to_eV=spec_nx.AA_to_eV
q_limit=spec_nx.q_limit
Buffer=spec_nx.Buffer

def SLD_calculations(z, item, sample, inst):
    res=spec_nx.SLD_calculations(z, item, sample, inst)
    res['z']-=5*refl.resolve_par(sample.Substrate, 'sigma')
    return res

SimulationFunctions={'Specular': spec_nx.Specular,
                     'SLD': SLD_calculations
                     }

(Instrument, Layer, Stack, Sample)=refl.MakeClasses(InstrumentParameters,
                                                    LayerParameters, StackParameters, SampleParameters,
                                                    SimulationFunctions,
                                                    ModelID)

def calculate_segmentation(sample):
    '''
      Calculate segmentation steps inside a sample defined by
      a maximum SLD slope and minimum step size. It first
      calculates the nuclear and magnetic SLD profile
      from the model and than separates it according to the
      given parameters.
    '''
    parameters=resolve_parameters_by_element(sample)
    dens=array(parameters['dens'], dtype=float64)
    f=array(parameters['f'], dtype=complex128)
    b=array(parameters['b'], dtype=complex128)
    xs_ai=array(parameters['xs_ai'], dtype=float64)
    magn=array(parameters['magn'], dtype=float64)
    magn_ang=array(parameters['magn_ang'], dtype=float64)/180.*pi
    magn_void=array(parameters['magn_void'], dtype=float64)

    sld_x=dens*f
    sld_n=dens*b
    sld_xs=dens*xs_ai
    # the new magnetization density and angle will be
    # calculated from perpendicular and parallel components to allow
    # a smooth transition in magnetic angle
    mag_sld_nsf=magn*dens*cos(magn_ang)
    mag_sld_sf=magn*dens*sin(magn_ang)

    d=array(parameters['d'], dtype=float64)
    d=d[1:-1]
    # Include one extra element - the zero pos (substrate/film interface)
    int_pos=cumsum(r_[0, d])
    rough_type=array(parameters['rough_type'], dtype=int)[:-1]
    sigma_n=array(parameters['sigma'], dtype=float64)[:-1]+1e-7
    sigma_m=array(parameters['sigma_mag'], dtype=float64)[:-1]+1e-7
    z=arange(-sigma_n[0]*5, int_pos.max()+sigma_n[-1]*5, sample.minimal_steps/5.0)
    # interface transition functions
    trans_n=(0.5-0.5*erf((z[:, newaxis]-int_pos)/sqrt(2.)/sigma_n))*(rough_type==0)
    trans_n+=maximum(0., minimum(1., (1.+(int_pos-z[:, newaxis])/2./sigma_n)/2.))*(rough_type==1)
    trans_n+=maximum(0., minimum(1., exp((int_pos-z[:, newaxis])/sigma_n)))*(rough_type==2)
    trans_n+=maximum(0., minimum(1., 1.-exp((z[:, newaxis]-int_pos)/sigma_n)))*(rough_type==3)
    trans_m=(0.5-0.5*erf((z[:, newaxis]-int_pos)/sqrt(2.)/sigma_m))*(rough_type==0)
    trans_m+=maximum(0., minimum(1., (1.+(int_pos-z[:, newaxis])/2./sigma_m)/2.))*(rough_type==1)
    trans_m+=maximum(0., minimum(1., exp((int_pos-z[:, newaxis])/sigma_m)))*(rough_type==2)
    trans_m+=maximum(0., minimum(1., 1.-exp((z[:, newaxis]-int_pos)/sigma_m)))*(rough_type==3)
    # SLD calculations
    rho_x=sum((sld_x[:-1]-sld_x[1:])*trans_n, 1)+sld_x[-1]
    rho_n=sum((sld_n[:-1]-sld_n[1:])*trans_n, 1)+sld_n[-1]
    rho_m_nsf=sum((mag_sld_nsf[:-1]-mag_sld_nsf[1:])*trans_m, 1)+mag_sld_nsf[-1]
    rho_m_sf=sum((mag_sld_sf[:-1]-mag_sld_sf[1:])*trans_m, 1)+mag_sld_sf[-1]
    rho_void=sum((magn_void[:-1]-magn_void[1:])*trans_n, 1)+magn_void[-1]
    xs_ai_comb=sum((sld_xs[:-1]-sld_xs[1:])*trans_n, 1)+sld_xs[-1]
    # add more elements to the SLDs
    for params in parameters['Elements'][1:]:
        dens=array(params['dens'], dtype=float64)
        f=array(params['f'], dtype=complex128)
        b=array(params['b'], dtype=complex128)
        xs_ai=array(params['xs_ai'], dtype=float64)
        magn=array(params['magn'], dtype=float64)
        magn_ang=array(params['magn_ang'], dtype=float64)/180.*pi
        sld_x=dens*f
        sld_n=dens*b
        sld_xs=dens*xs_ai
        mag_sld_nsf=magn*dens*cos(magn_ang)
        mag_sld_sf=magn*dens*sin(magn_ang)

        d=array(params['d'], dtype=float64)
        d=d[1:-1]
        # Include one extra element - the zero pos (substrate/film interface)
        int_pos=cumsum(r_[0, d])
        rough_type=array(params['rough_type'], dtype=int)[:-1]
        sigma_n=array(params['sigma'], dtype=float64)[:-1]+1e-7
        sigma_m=array(params['sigma_mag'], dtype=float64)[:-1]+1e-7
        # interface transition functions
        trans_n=(0.5-0.5*erf((z[:, newaxis]-int_pos)/sqrt(2.)/sigma_n))*(rough_type==0)
        trans_n+=maximum(0., minimum(1., (1.+(int_pos-z[:, newaxis])/2./sigma_n)/2.))*(rough_type==1)
        trans_n+=maximum(0., minimum(1., exp((int_pos-z[:, newaxis])/sigma_n)))*(rough_type==2)
        trans_n+=maximum(0., minimum(1., 1.-exp((z[:, newaxis]-int_pos)/sigma_n)))*(rough_type==3)
        trans_m=(0.5-0.5*erf((z[:, newaxis]-int_pos)/sqrt(2.)/sigma_m))*(rough_type==0)
        trans_m+=maximum(0., minimum(1., (1.+(int_pos-z[:, newaxis])/2./sigma_m)/2.))*(rough_type==1)
        trans_m+=maximum(0., minimum(1., exp((int_pos-z[:, newaxis])/sigma_m)))*(rough_type==2)
        trans_m+=maximum(0., minimum(1., 1.-exp((z[:, newaxis]-int_pos)/sigma_m)))*(rough_type==3)
        # SLD calculations
        rho_x+=sum((sld_x[:-1]-sld_x[1:])*trans_n, 1)+sld_x[-1]
        rho_n+=sum((sld_n[:-1]-sld_n[1:])*trans_n, 1)+sld_n[-1]
        rho_m_nsf+=sum((mag_sld_nsf[:-1]-mag_sld_nsf[1:])*trans_m, 1)+mag_sld_nsf[-1]
        rho_m_sf+=sum((mag_sld_sf[:-1]-mag_sld_sf[1:])*trans_m, 1)+mag_sld_sf[-1]
        xs_ai_comb+=sum((sld_xs[:-1]-sld_xs[1:])*trans_n, 1)+sld_xs[-1]
    # calculate the segmentation
    d_segments=[0.]
    i=0
    rho_x_out=[rho_x[0]]
    rho_n_out=[rho_n[0]]
    rho_nsf_out=[rho_m_nsf[0]]
    rho_sf_out=[rho_m_sf[0]]
    xs_ai_out=[xs_ai_comb[0]]
    while i<(len(z)-1):
        # calculate SLD differeces for all subsequent positions
        #        diff_x=abs(rho_x[i+1:]-rho_x_out[-1])
        #        diff_n_p=abs(rho_n[i+1:]+rho_m_nsf[i+1:]-rho_n_out[-1]-rho_nsf_out[-1])
        #        diff_n_m=abs(rho_n[i+1:]-rho_m_nsf[i+1:]-rho_n_out[-1]+rho_nsf_out[-1])
        #        diff_m_sf=abs(rho_m_sf[i+1:]-rho_sf_out[-1])
        #        diff_idx=where((diff_n_p>sample.max_diff_n)|(diff_x>sample.max_diff_x)|
        #                       (diff_n_m>sample.max_diff_n)|(diff_m_sf>sample.max_diff_n))[0]
        # calculate the maximum variation of SLD up to given index
        diff_x=abs(maximum.accumulate(rho_x[i+1:])-minimum.accumulate(rho_x[i+1:]))
        diff_n_p=abs(maximum.accumulate(rho_n[i+1:]+rho_m_nsf[i+1:])-minimum.accumulate(rho_n[i+1:]+rho_m_nsf[i+1:]))
        diff_n_m=abs(maximum.accumulate(rho_n[i+1:]-rho_m_nsf[i+1:])-minimum.accumulate(rho_n[i+1:]-rho_m_nsf[i+1:]))
        diff_m_sf=abs(maximum.accumulate(rho_m_sf[i+1:])-minimum.accumulate(rho_m_sf[i+1:]))
        diff_idx=where(logical_not((diff_n_p<sample.max_diff_n) & (diff_x<sample.max_diff_x) &
                                   (diff_n_m<sample.max_diff_n) & (diff_m_sf<sample.max_diff_n)))[0]
        if len(diff_idx)>0:
            j=min(len(z)-1, max(i+diff_idx[0]+1, i+5))
        else:
            j=len(z)-1  # last position
        d_segments.append(z[j]-z[i])
        rho_x_out.append(rho_x[i:j].mean())
        rho_n_out.append(rho_n[i:j].mean())
        rho_nsf_out.append(rho_m_nsf[i:j].mean()*(1.-rho_void[i:j].mean()))  # averadge magn taking voids into account
        rho_sf_out.append(rho_m_sf[i:j].mean()*(1.-rho_void[i:j].mean()))  # averadge magn taking voids into account
        xs_ai_out.append(xs_ai_comb[i:j].mean())  # averadge mang angle
        i=j
    rho_nsf_out=array(rho_nsf_out)
    rho_sf_out=array(rho_sf_out)
    rho_m_out=sqrt(rho_nsf_out**2+rho_sf_out**2).tolist()
    magn_ang_out=(arctan2(rho_nsf_out, -rho_sf_out)*180./pi-90.).tolist()
    return (d_segments[1:], rho_x_out[1:],
            rho_n_out[1:], rho_m_out[1:], xs_ai_out[1:], magn_ang_out[1:])

def resolve_parameters_by_element(sample):
    """
      Resolve the model standard parameters for each element.
      The first element used for normal parameter names,
      every other element does ignore substrate and ambience sld
      and is assigned to the 'elements' keyword as list.
      This makes it possible to build SLD profiles as sum of all
      elements.
    """
    elements=list(set([stack.Element for stack in sample.Stacks]))
    elements.sort()
    par=sample.Substrate._parameters.copy()
    for k in par:
        par[k]=[refl.resolve_par(sample.Substrate, k)]
    for k in sample.Substrate._parameters:
        for stack in sample.Stacks:
            if stack.Element!=0:
                continue
            par[k]=par[k]+stack.resolveLayerParameter(k)
        par[k]=par[k]+[refl.resolve_par(sample.Ambient, k)]
    par['sigma_mag']=where(array(par['sigma_mag'])!=0.,
                           par['sigma_mag'], par['sigma']).tolist()
    output=par
    output['Elements']=[]
    for element in elements:
        par=sample.Substrate._parameters.copy()
        for k in list(par.keys()):
            par[k]=[refl.resolve_par(sample.Substrate, k)]
        # zero substrat SLD
        par['f']=[0j]
        par['b']=[0j]
        par['magn']=[0.]
        par['xs_ai']=[0.]
        par['magn_ang']=[0.]
        for k in sample.Substrate._parameters:
            for stack in sample.Stacks:
                if stack.Element!=element:
                    continue
                par[k]=par[k]+stack.resolveLayerParameter(k)
            par[k]=par[k]+[refl.resolve_par(sample.Ambient, k)+0.0]
        # zero ambience SLD for Element
        par['f'][-1]=0j
        par['b'][-1]=0j
        par['magn'][-1]=0.
        par['xs_ai'][-1]=0.
        par['magn_ang'][-1]=0.
        # if magnetic roughness is set to zero use structural roughness
        par['sigma_mag']=where(array(par['sigma_mag'])!=0.,
                               par['sigma_mag'], par['sigma']).tolist()
        output['Elements'].append(par)
    return output

def resolveLayerParameters(self):
    # resolve parameters by creating layers automatically
    # adapting changes to the SLDs
    par={}
    for k in ['b', 'd', 'dens', 'f', 'magn', 'magn_ang', 'sigma', 'xs_ai']:
        par[k]=[refl.resolve_par(self.Substrate, k)]
    par['sigma'][0]=0.
    d, rho_x, rho_n, rho_m, xs_ai, magn_ang=calculate_segmentation(self)
    par['d']+=d
    par['f']+=rho_x
    par['b']+=rho_n
    par['magn']+=rho_m
    par['xs_ai']+=xs_ai
    if self.smoothen:
        par['sigma']+=[(d[i]+d[i+1])/4. for i in range(len(d)-1)]+[0.]
    else:
        par['sigma']+=[0. for ignore in d]
    par['magn_ang']+=magn_ang
    par['dens']+=[1. for ignore in d]
    for k in ['b', 'd', 'dens', 'f', 'magn', 'magn_ang', 'sigma', 'xs_ai']:
        par[k].append(refl.resolve_par(self.Ambient, k))
    return par

Sample.resolveLayerParameters=resolveLayerParameters

if __name__=='__main__':
    pass



# -*- coding: utf8 -*-
'''<h1>Library for combined x-ray and neutrons simulations for inhomogeneous samples.</h1>
<p>Most of the model is the same as in spec_nx.</p>
'''
from copy import deepcopy
from numpy import *
from scipy.special import wofz
from . import spec_nx
from .spec_nx import refl
from .lib.instrument import *

# Preamble to define the parameters needed for the models outlined below:
ModelID='SpecInhom'
__pars__=spec_nx.__pars__.copy()

instrument_string_choices=spec_nx.instrument_string_choices.copy()
InstrumentParameters=spec_nx.InstrumentParameters.copy()
InstrumentGroups=spec_nx.InstrumentGroups.copy()
InstrumentUnits=spec_nx.InstrumentUnits.copy()

LayerParameters=spec_nx.LayerParameters.copy()
LayerParameters.update({'sigma_gradient': 0.0, 'd_gradient': 0.0})
LayerUnits=spec_nx.LayerUnits.copy()
LayerUnits.update({'sigma_gradient': '1', 'd_gradient': '1'})
LayerGroups=[('Standard', ['f', 'dens', 'd', 'sigma']),
             ('Neutron', ['b', 'xs_ai', 'magn', 'magn_ang']),
             ('Inhom.', ['sigma_gradient', 'd_gradient'])]

StackParameters={'Layers': [], 'Repetitions': 1,
                 'sigma_gradient': 0.,  # amount of increase in roughness from bottom to top
                 'sigma_gtype': 3,  # model for this increas (rel lin, rel sqrt, abs lin, abs sqrt)
                 'd_gradient': 0., 'dens_gradient': 0.,  # change in thickness or density bottom to top
                 }
SampleParameters=spec_nx.SampleParameters.copy()
SampleParameters.update({'sigma_inhom': 0.0, 'lscale_inhom': 0.9, 'flatwidth_inhom': 0.3,
                         'steps_inhom': 20, 'type_inhom': 'empiric PLD'})
sample_string_choices={
    'type_inhom': ['gauss', 'semi-gauss', 'empiric PLD'],
    }

AA_to_eV=spec_nx.AA_to_eV
q_limit=spec_nx.q_limit
Buffer=spec_nx.Buffer

def Specular(TwoThetaQz, sample, instrument):
    '''
      The model function. Averadging the intensities for different
      layer thicknesses as found for e.g. large PLD samples.
    '''

    Q, TwoThetaQz, weight=spec_nx.resolution_init(TwoThetaQz, instrument)
    if any(Q<q_limit):
        raise ValueError('The q vector has to be above %.1e'%q_limit)
    restype=instrument.getRestype()
    foottype=instrument.getFootype()
    Ibkg=instrument.getIbkg()
    instrument.setRestype('no conv')
    instrument.setFootype('no corr')
    instrument.Ibkg=0.
    sampcall=deepcopy(sample)

    # average thicknesses before inhomogeniety average
    d0=[array([Layer.getD() for Layer in Stack.Layers]) for Stack in sample.Stacks]
    sigma_d=sample.getSigma_inhom()*0.01  # Inhomogeniety in \% (gamma for type 2)
    lorentz_scale=sample.getLscale_inhom()
    flat_width=maximum(1e-4, sample.getFlatwidth_inhom()*0.01)
    type_inhom=sample.getType_inhom()
    # Define the thicknesses to calculate and their propability
    if type_inhom in sample_string_choices['type_inhom']:
        type_inhom=sample_string_choices['type_inhom'].index(type_inhom)
    else:
        type_inhom=0
    if sigma_d==0:  # no inhomogeniety
        d_fact=array([1.])
        P=array([1.])
    elif type_inhom==1:  # half gaussian shape inhomogeniety
        d_fact=1.+linspace(-2.*sigma_d, 0, sample.getSteps_inhom())
        P=exp(-0.5*(d_fact-sigma_d-1.)**2/sigma_d**2)
        P/=P.sum()
        mean_d=(P*d_fact).sum()
        d_fact+=1.-mean_d
    elif type_inhom==2:  # inhomogeniety of a PLD line focus, better approximation
        d_fact=1.+linspace(-1.*max(2.*sigma_d, flat_width), 0, sample.getSteps_inhom())
        Pg=where(d_fact>flat_width, lorentz_scale*1./(1.+((d_fact-1.)/sigma_d)**2), 0.)
        Pf=(1.-lorentz_scale)*where(d_fact>flat_width, 1., 0.)
        P=Pg+Pf
        P/=P.sum()
        mean_d=(P*d_fact).sum()
        d_fact+=1.-mean_d
    else:  # gaussian inhomegeniety
        d_fact=1.+linspace(-sigma_d, sigma_d, sample.getSteps_inhom())
        P=exp(-0.5*(d_fact-1.)**2/sigma_d**2)
        P/=P.sum()
    # list for reflectivities to average
    Rlist=[]
    # Iterate over thicknesses
    for d_facti, Pi in zip(d_fact, P):
        di=[d_facti*d0i for d0i in d0]
        for i, Stack in enumerate(sampcall.Stacks):
            for j, Layer in enumerate(Stack.Layers):
                # Layer.setD(di[i][j])
                Layer.d=di[i][j]
        Rlist.append(Pi*spec_nx.Specular(TwoThetaQz, sampcall, instrument))
    R=array(Rlist).sum(axis=0)

    instrument.setRestype(restype)
    instrument.setFootype(foottype)
    instrument.Ibkg=Ibkg

    # footprint correction
    foocor=spec_nx.footprintcorr(Q, instrument)
    # resolution correction
    R=spec_nx.resolutioncorr(R, TwoThetaQz, foocor, instrument, weight)
    return R+instrument.getIbkg()

def ResolutionVectorAsymetric(Q, dQ, points, dLambda, asymmetry, range_=3):
    '''
      Resolution vector for a asymmetric wavelength distribution found in
      neutron experiments with multilayer monochromator.
    '''
    Qrange=max(range_*dQ, range_*dLambda*Q.max())
    Qstep=2*Qrange/points
    Qres=Q+(arange(points)-(points-1)/2)[:, newaxis]*Qstep
    Quse=transpose(Q[:, newaxis])

    gamma_asym=2.*dLambda*Quse/(1+exp(asymmetry*(Quse-Qres)))
    z=(Quse-Qres+(abs(gamma_asym)*1j))/abs(dQ)/sqrt(2.)
    z0=(0.+(abs(gamma_asym)*1j))/abs(dQ)/sqrt(2)
    weight=wofz(z).real/wofz(z0).real
    Qret=Qres.flatten()
    return Qret, weight

SLD_calculations=spec_nx.SLD_calculations

SimulationFunctions={'Specular': Specular,
                     'SLD': SLD_calculations
                     }

(Instrument, Layer, Stack, Sample)=refl.MakeClasses(InstrumentParameters,
                                                    LayerParameters, StackParameters, SampleParameters,
                                                    SimulationFunctions,
                                                    ModelID)

# Add gradient for sigma and thickness to multilayers
def resolveLayerParameter(self, parameter):
    if parameter=='sigma':
        sigma_gradient=self.sigma_gradient
        # parameters for layers with roughness gradient
        par=[refl.resolve_par(lay, parameter) for lay in self.Layers]
        for i in range(1, self.Repetitions):
            if self.sigma_gtype==0:
                # linear increase of roughness to (1+sigma_gradient) times bottom roughness)
                par+=[refl.resolve_par(lay, parameter)*(1.+(sigma_gradient+lay.sigma_gradient)*i/(self.Repetitions-1))
                      for lay in self.Layers]
            elif self.sigma_gtype==1:
                # add roughness using rms
                par+=[refl.resolve_par(lay, parameter)*sqrt(
                    1.+((sigma_gradient+lay.sigma_gradient)*i/(self.Repetitions-1))**2)
                      for lay in self.Layers]
            elif self.sigma_gtype==2:
                # linear increase of roughness to bottom roughness + sigma_gradient)
                par+=[refl.resolve_par(lay, parameter)+(sigma_gradient+lay.sigma_gradient)*i/(self.Repetitions-1)
                      for lay in self.Layers]
            elif self.sigma_gtype==3:
                # add roughness using rms
                par+=[sqrt(
                    refl.resolve_par(lay, parameter)**2+((sigma_gradient+lay.sigma_gradient)*i/(self.Repetitions-1))**2)
                      for lay in self.Layers]
            else:
                raise NotImplementedError('sigma_gtype must be between 0 and 3')
    elif parameter=='d':
        d_gradient=self.d_gradient
        # parameters for layers with roughness gradient
        par=[]
        for i in range(self.Repetitions):
            par+=[refl.resolve_par(lay, parameter)*(1.-(d_gradient+lay.d_gradient)*(1./2.-float(i)/self.Repetitions))
                  for lay in self.Layers]
    elif parameter in ['dens']:
        dens_gradient=self.dens_gradient
        # parameters for layers with roughness gradient
        par=[]
        for i in range(self.Repetitions):
            par+=[refl.resolve_par(lay, parameter)*(1.-dens_gradient*(1./2.-float(i)/self.Repetitions)) for lay in
                  self.Layers]
    else:
        par=[refl.resolve_par(lay, parameter)+0.0 for lay in self.Layers]*self.Repetitions
    return par

Stack.resolveLayerParameter=resolveLayerParameter



"""
File reader and writer for ORSO text file standard .ort
"""
import re
import numpy as np
import json
from datetime import datetime
from collections import OrderedDict
from . import ORSOIOError
from .data import ORSOData

# for keeping datetime as a string (schema validation)
def yaml_timestamp_constructor(loader, node):
    return str(node.value)

def parse_yaml(header):
    import yaml
    try:
        from yaml import CLoader as Loader
    except ImportError:
        from yaml import Loader

    class StringDatetimeLoader(Loader):
        pass

    StringDatetimeLoader.add_constructor(u'tag:yaml.org,2002:timestamp', yaml_timestamp_constructor)

    return yaml.load(header, Loader=StringDatetimeLoader)

# in case we want to represent datetime in python
# def date_hook(json_dict):
#     for (key, value) in json_dict.items():
#         try:
#             json_dict[key] = datetime.strptime(value, "%Y-%m-%dT%H:%M:%S")
#         except:
#             pass
#     return json_dict

def parse_json(header):
    return json.loads(header)  # , object_hook=date_hook)

def read_data(text_data):
    # read the data from the text, faster then numpy loadtxt with StringIO
    data=[li.split() for li in text_data.strip().splitlines() if not li.startswith('#')]
    return np.array(data, dtype=float).T

def recursive_update(main, sub):
    # create a dictionary that replaces every item from main with sub, if it exists
    # includes sub-key updates
    output=main.copy()
    for key, value in sub.items():
        if not key in output:
            output[key]=value
        elif type(value) is dict:
            output[key]=recursive_update(main[key], value)
        else:
            output[key]=value
    return output

def read_file(fname):
    with open(fname, 'r') as fh:
        # check if this is the right file type
        l1=fh.readline()
        if not l1.lower().startswith('# orso'):
            raise ORSOIOError('Not an ORSO reflectivity text file, wrong header')
        # find end of comment block (first line not starting with #
        text=fh.read()
    ftype_info=list(map(str.strip, l1.split('|')))
    data_start=re.search(r'\n[^#]', text).start()
    header=text[:data_start-1].rsplit('\n', 1)[0]  # remove last header line that only contains column info
    header=header[2:].replace('\n# ', '\n')  # remove header comment to make the text valid yaml/json
    header_encoding=ftype_info[2].lower().split()[0]
    if header_encoding=='yaml':
        main_header=parse_yaml(header)
        ds_string='\n# data_set:'
    elif header_encoding=='json':
        main_header=parse_json(header)
        ds_string='\n# {\n#     "data_set":'
    else:
        raise ORSOIOError('Unknown header encoding "%s"'%header_encoding)

    # implement possibility of more then one data block:
    if ds_string in text:
        split_indices=[match.start()+data_start for match in re.finditer(ds_string, text[data_start:])]+[-1]
        output=[ORSOData(main_header, read_data(text[data_start:split_indices[0]]), strict=False)]
        for i, si in enumerate(split_indices[:-1]):
            sub_header_length=re.search(r'\n[^#]', text[si:]).start()
            data=read_data(text[si+sub_header_length:split_indices[i+1]])
            sub_header_text=text[si+2:si+sub_header_length].rsplit('\n', 1)[0].replace('\n# ', '\n').strip()
            if header_encoding=='yaml':
                sub_header_data=parse_yaml(sub_header_text)
            elif header_encoding=='json':
                sub_header_data=parse_json(sub_header_text)
            # create a merged dictionary
            sub_headers_dict=recursive_update(main_header, sub_header_data)
            output.append(ORSOData(sub_headers_dict, data, strict=False))
        return output
    else:
        data=read_data(text[data_start:])
        return ORSOData(main_header, data, strict=False)

def encode_yaml(header):
    import yaml
    def _dict_representer(dumper, data):
        return dumper.represent_mapping(
            yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
            data.items())

    try:
        from yaml import CDumper as Dumper
    except ImportError:
        from yaml import Dumper

    class DumperInd(yaml.Dumper):
        # Dumper the fixes indentation of list items
        def increase_indent(self, flow=False, *args, **kwargs):
            return super().increase_indent(flow=flow, indentless=False)

    DumperInd.add_representer(OrderedDict, _dict_representer)
    if 'columns' in header:
        header=header.copy()
        columns=header['columns']
        del (header['columns'])
        output=yaml.dump(header, Dumper=DumperInd,
                         indent=4, width=78, sort_keys=False)
        output+='columns:\n'
        for col in columns:
            output+='    - %s'%yaml.dump(col, Dumper=DumperInd,
                                         indent=4, width=72, sort_keys=False, default_flow_style=True)
    else:
        output=yaml.dump(header, Dumper=DumperInd, indent=4, width=78, sort_keys=False)
    return output

def encode_json(header):
    # in case we want to represent date as datetime in python
    # class DateEncoder(json.JSONEncoder):
    #     def default(self, o):
    #         if type(o)==datetime:
    #             return o.strftime('%Y-%m-%dT%H:%M:%S')
    #         else:
    #             return json.JSONEncoder.default(self, o)
    # return json.dumps(header, indent=4, cls=DateEncoder)+'\n'
    return json.dumps(header, indent=4)+'\n'

def dict_change(main, new):
    # get a dictionary with any changed or new items including sub-dictionaries
    output=OrderedDict({})
    for mkey in main.keys():
        if mkey in new and main[mkey]!=new[mkey]:
            item=new[mkey]
            if type(item) is dict:
                output[mkey]=dict_change(main[mkey], item)
            else:
                output[mkey]=item
    for nkey in new.keys():
        if not nkey in main:
            output[nkey]=new[nkey]
    return output

def data_header(columns):
    output="%-15s"%("1 %s"%columns[0])+' '.join(["%-16s"%("%i %s"%(i+2, ci)) for i, ci in enumerate(columns[1:])])
    return output

def write_file(fname, data, header_encoding='yaml', force_ending=True):
    if force_ending and not fname.endswith('.ort'):
        fname=fname+'.ort'
    if type(data) is list:
        main_dict=data[0].header
        if header_encoding=='yaml':
            header='ORSO reflectivity data file | 0.1 standard | YAML encoding | https://www.reflectometry.org/\n'
            header+=encode_yaml(main_dict)
        elif header_encoding=='json':
            header='ORSO reflectivity data file | 0.1 standard | JSON encoding | https://www.reflectometry.org/\n'
            header+=encode_json(main_dict)
        else:
            raise ORSOIOError('Unknown header encoding "%s"'%header_encoding)
        with open(fname, 'w') as fh:
            header+=data_header(data[0].columns)
            np.savetxt(fh, data[0].data.T, header=header, fmt='%-16.9e')
            for i, di in enumerate(data[1:]):
                fh.write('\n')
                update_dict=dict_change(main_dict, di.header)
                if not 'data_set' in update_dict:
                    update_dict['data_set']='data_set'
                update_dict.move_to_end('data_set', False)
                if header_encoding=='yaml':
                    header=encode_yaml(update_dict)
                elif header_encoding=='json':
                    header=encode_json(update_dict)
                header+=data_header(di.columns)
                np.savetxt(fh, di.data.T, header=header, fmt='%-16.9e')
    else:
        main_dict=data.header
        if header_encoding=='yaml':
            header='ORSO reflectivity data file | 0.1 standard | YAML encoding | https://www.reflectometry.org/\n'
            header+=encode_yaml(main_dict)
        elif header_encoding=='json':
            header='ORSO reflectivity data file | 0.1 standard | JSON encoding | https://www.reflectometry.org/\n'
            header+=encode_json(main_dict)
        else:
            raise ORSOIOError('Unknown header encoding "%s"'%header_encoding)
        header+=data_header(data.columns)
        np.savetxt(fname, data.data.T, header=header, fmt='%-16.9e')
